import urllib
import dateutil.parser
from datetime import datetime
from pathlib import Path
from typing import List, Union, Type
from lodstorage.lod import LOD
from lodstorage.storageconfig import StorageConfig
from wikibot.wikipush import WikiPush
from wikibot.wikiuser import WikiUser
from wikifile.wikiFile import WikiFile
from wikifile.wikiFileManager import WikiFileManager
from wikifile.wikiPage import WikiPage

from corpus.event import Event, EventSeries, EventSeriesManager, EventManager
from corpus.eventcorpus import EventDataSource,EventDataSourceConfig
from corpus.utils.download import Profiler
from ptp.ordinal import Ordinal


class OR(EventDataSource):
    '''
    scientific events from http://www.openresearch.org
    '''

    limitFiles = None
    
    def __init__(self,wikiId='or',via='api'):
        '''
        constructor
        
        Args:
            wikiId(str): the wikiId to get the SMW data from 
            via(str): the access style api or backup
        '''
        lookupId=f"{wikiId}" if via=="api" else f"{wikiId}-backup"
        tableSuffix=f"{wikiId}" if via=="api" else f"{wikiId}backup"
        name=f"{wikiId}-{via}"
        title=f"OPENRESEARCH ({wikiId}-{via})"   
        wikiUser=None 
        try:
            wikiUser=WikiUser.ofWikiId(wikiId, lenient=True)
        except FileNotFoundError as _fne:
            # configuration file missing
            pass
        if wikiUser is not None:
            url=wikiUser.getWikiUrl()
        else:
            if wikiId == "or":
                url = 'https://www.openresearch.org/wiki/Main_Page'
            else:
                url = "https://confident.dbis.rwth-aachen.de/or/index.php?title=Main_Page"
        sourceConfig = EventDataSourceConfig(lookupId=lookupId,
                                             name=name,
                                             url=url,
                                             title=title,
                                             tableSuffix=tableSuffix,
                                             locationAttribute='location')
        eventManager = OREventManager(sourceConfig=sourceConfig, wikiId=wikiId, via=via)
        seriesManager = OREventSeriesManager(sourceConfig=sourceConfig, wikiId=wikiId, via=via)
        super().__init__(eventManager, seriesManager, sourceConfig)


class OREventManager(EventManager):
    '''
    Manager for OpenResearch Events
    
    see https://www.openresearch.org
    '''

    def __init__(self,
                 sourceConfig:EventDataSourceConfig=None,
                 config:StorageConfig=None,
                 wikiId:str='orclone',
                 via:str='api',
                 verbose:bool=False,
                 debug:bool=False):
        '''
        Constructor

        Args:
            sourceConfig(EventDataSourceConfig): event source configuration
            config(StorageConfig): storage configuration
            wikiId(str): wikiId of the event source wiki
            via(str): method the data is retrieved from the api
            verbose(bool):
            debug(bool): If True show debug messages
        '''
        self.events=[]
        super().__init__(name="OREvents",
                         sourceConfig=sourceConfig,
                         clazz=OREvent,
                         primaryKey="pageTitle",
                         config=config)
        self.wikiId = wikiId
        self.via = via
        self.debug=debug
        self.verbose=verbose
        if self.debug:
            self.profile = True
    
    def configure(self):
        '''
        configure me
        '''
        if not hasattr(self, "getListOfDicts"):
            if self.via == 'wikiMarkup':
                self.getListOfDicts = self.getLodFromWikiMarkup
            elif self.via == 'backup':
                self.getListOfDicts = self.getLoDfromWikiFileManager
            else:
                self.getListOfDicts = self.getLoDfromWikiUser

    def getLodFromWikiMarkup(self, limit:int=None) -> List[dict]:
        """
        Retrieves the event records from the WikiMarkup of the event pages.

        Args:
            limit(int): limit number of retrieved records. If None (default) retrieve all event records

        Returns:
            LoD List of event records
        """
        lod = OrSMW.getLodFromWikiMarkup(self.wikiId, OREvent, limit=limit)
        self.setAllAttr(lod, "source", f"{self.wikiId}-wikiMarkup")
        self.postProcessLodRecords(lod, wikiId=self.wikiId, debug=self.debug)
        return lod

    def getLoDfromWikiUser(self, askExtra:str="", profile:bool=False, limit:int=None):
        '''

        Args:
            wikiuser(WikiUser):
            askExtra(str):
            profile(bool):
        '''
        lod = OrSMW.getLodFromWikiApi(self.wikiId, OREvent, askExtra=askExtra, limit=limit, profile=profile)
        self.setAllAttr(lod,"source",f"{self.wikiId}")
        self.postProcessLodRecords(lod,wikiId=self.wikiId, debug=self.debug)
        return lod

    def getLoDfromWikiFileManager(self, wikiFileManager:WikiFileManager=None, limit:int=None):
        '''
        get my list of dicts from the given WikiFileManager

        Args:
            wikiFileManager(WikiFileManager): WikiFileManager from which the records should be loaded
            limit(int): limit the amount on loaded records
        '''
        lod = OrSMW.getLodFromWikiFiles(self.wikiId, OREvent, wikiFileManager=wikiFileManager, limit=limit)
        self.setAllAttr(lod,"source",f"{self.wikiId}-backup")
        self.postProcessLodRecords(lod,wikiId=self.wikiId, debug=self.debug)
        return lod

    @classmethod
    def getPropertyLookup(cls) -> dict:
        '''
        get my PropertyLookupList as a map

        Returns:
            dict: my mapping from wiki property names to LoD attribute Names or None if no mapping is defined
        '''
        lookup = None
        if hasattr(OREvent, 'propertyLookupList'):
            propertyLookupList = OREvent.propertyLookupList
            lookup, _duplicates = LOD.getLookup(propertyLookupList, 'prop')
        return lookup

    def fromWikiUser(self, askExtra: str = "", limit:int=None, profile: bool = False) -> List[dict]:
        '''
        read me from a wiki using the given WikiUser configuration

        Args:
            askExtra(string): Extra query selectors that should be included in the query
            limit(int): limit the amount on loaded records
            profile(bool): If true profile the query. Otherwise, the query runs without tracking the time
        '''
        lod = self.getLoDfromWikiUser(askExtra, profile=profile, limit=limit)
        self.fromLoD(lod)
        return lod

    def fromWikiFileManager(self, wikiFileManager: WikiFileManager=None, limit:int=None) -> List[dict]:
        '''
        read me from wiki markup files using the given WikiFileManager

        Args:
            wikiFileManager(WikiFileManager): WikiFileManager to parse the wiki markup files
            limit(int): limit the amount on loaded records
        '''
        lod = self.getLoDfromWikiFileManager(wikiFileManager, limit=limit)
        self.fromLoD(lod)
        return lod


class OREvent(Event):
    '''
    I represent an Event retrieved from OPENRESEARCH

    see https://rq.bitplan.com/index.php/Event
    '''
    templateName = "Event"
    entityName = 'Event'

    propertyLookupList = [
        {'prop': 'Acronym', 'name': 'acronym', 'templateParam': "Acronym"},
        {'prop': 'End date', 'name': 'endDate', 'templateParam': "End date"},
        {'prop': 'Event in series', 'name': 'inEventSeries', 'templateParam': "Series"},
        {'prop': 'Event presence', 'name': 'presence', 'templateParam': "presence"},
        {'prop': 'Event type', 'name': 'eventType', 'templateParam': "Type"},
        {'prop': 'Has_location_country', 'name': 'country', 'templateParam': "Country"},
        {'prop': 'Has_location_state', 'name': 'region', 'templateParam': "State"},
        {'prop': 'Has_location_city', 'name': 'city', 'templateParam': "City"},
        {'prop': 'Has year', 'name': 'year', 'templateParam': "Year"},
        {'prop': 'Homepage', 'name': 'homepage', 'templateParam': "Homepage"},
        {'prop': 'Ordinal', 'name': 'ordinal', 'templateParam': "Ordinal"},
        {'prop': 'Start date', 'name': 'startDate', 'templateParam': "Start date"},
        {'prop': 'Title', 'name': 'title', 'templateParam': "Title"},
        {'prop': 'Accepted papers', 'name': 'acceptedPapers', 'templateParam': "Accepted papers"},
        {'prop': 'Submitted papers', 'name': 'submittedPapers', 'templateParam': "Submitted papers"},
        {'prop': 'presence', 'name': 'presence', 'templateParam': "presence"},
        {'prop': 'WikiCFP-ID', 'name': 'wikicfpId', 'templateParam': 'wikicfpId' },
        {'prop':'DblpConferenceId','name':'DblpConferenceId','templateParam':'DblpConferenceId'},
        {'prop': 'TibKatId', 'name': 'TibKatId', 'templateParam': 'TibKatId'},
        {'prop': 'TIBKAT-ISBN', 'name': 'ISBN', 'templateParam': 'ISBN'},
        {'prop': 'Wikidataid', 'name': 'wikidataId', 'templateParam': 'wikidataid'},
        {'prop': 'GND-ID', 'name': 'gndId', 'templateParam': 'gndId'}
    ]

    def __init__(self):
        '''
        Constructor
        '''
        super().__init__()

    @classmethod
    def getSamples(cls):
        samplesLOD = [{
                "pageTitle": "ICSME 2020",
                "acronym": "ICSME 2020",
                "ordinal": 36,
                "eventType": "Conference",
                "startDate": datetime.fromisoformat("2020-09-27"),
                "endDate": datetime.fromisoformat("2020-09-27")
            },
            {
                "pageTitle": "WebSci 2019",
                "acronym": "WebSci 2019",
                "ordinal": 11,
                "homepage": "http://websci19.webscience.org/",
                "title": "11th ACM Conference on Web Science",
                "eventType": "Conference",
                "startDate": datetime.fromisoformat("2019-06-30"),
                "endDate": datetime.fromisoformat("2019-07-03"),
                "inEventSeries": "WebSci",
                "country": "USA",
                "region": "US-MA",
                "city": "Boston",
                "acceptedPapers": 41,
                "submittedPapers": 120,
                "presence": "online",
                "wikicfpId": 891,
                "tibKatId":"1736060724",
                "subject": "Software engineering",
                "ISBN":"9781450370707",
                "gndId":"1221636014"
            },
            {
                "acronym": "5GU 2017",
                "city": "Melbourne",
                "country": "Australia",
                "endDate": datetime.fromisoformat("2017-06-09T00:00:00"),
                "eventType": "Conference",
                "homepage": "http://5guconference.org/2017/show/cf-papers",
                "inEventSeries": "5GU",
                "ordinal": 2,
                "startDate": datetime.fromisoformat("2017-06-08T00:00:00"),
                "title": "2nd EAI International Conference on 5G for Ubiquitous Connectivity",
                # technical attributes - SMW specific
                "pageTitle": "5GU 2017",
                "lastEditor": "Wolfgang Fahl",
                "creationDate": datetime.fromisoformat("2016-09-25T07:36:02"),
                "modificationDate": datetime.fromisoformat("2020-11-05T12:33:23")
            },
            {
                'acronym': "IDC 2009",
                'title': "The 8th International Conference on Interaction Design and Children",
                'pageTitle': 'IDC 2009',
                'ordinal': 8,
                'year':2009,
                'yearStr':'2009',
                'url':'https://confident.dbis.rwth-aachen.de/or/index.php?title=IDC_2009'
            },
            {
                'acronym': "VNC 2019",
                'title': "2019 IEEE Vehicular Networking Conference (VNC)",
                'pageTitle': 'VNC 2019',
                'ordinal': 11,
                'DblpConferenceId':"vnc/vnc2019",
                'wikidataid':"Q106335329",
                'doi':'10.1109/VNC48660.2019'
            }

        ]
        return samplesLOD

    @classmethod
    def getSampleWikiTextList(cls, mode='legacy'):
        if mode == 'legacy':
            samplesWikiSon = ["""{{Event
|Acronym=ICSME 2020
|Title=36th IEEE International Conference on Software Maintenance and Evolution
|Ordinal=36
|Series=ICSME
|Type=Conference
|Field=Software engineering
|Start date=2020/09/27
|End date=2020/10/03
|Homepage=https://icsme2020.github.io/
|City=Adelaide
|presence=online
|Country=Australia
|Abstract deadline=2020/05/22
|Paper deadline=2020/05/28
|Notification=2020/08/04
|Camera ready=2020/08/25
|Has host organization=Institute of Electrical and Electronics Engineers
|Has coordinator=Sebastian Baltes
|has general chair=Christoph Treude, Hongyu Zhang
|has program chair=Kelly Blincoe, Zhenchang Xing
|has demo chair=Mario Linares Vasquez, Hailong Sun
}}''',
'''36th IEEE International Conference on Software Maintenance and Evolution (ICSME)'''""", """{{Event
|Acronym=AISB 2009
|Ordinal=36
|Title=AISB Symposium: New Frontiers in Human-Robot Interaction
|Type=Conference
|Field=Uncategorized
|Start date=2009/04/08
|End date=2009/04/09
|Submission deadline=2009/01/05
|Homepage=homepages.feis.herts.ac.uk/~comqkd/HRI-AISB2009-Symposium.html
|City=Edinburgh
|Country=United Kingdom
|Notification=2009/02/02
|Camera ready=2009/02/23
}}	
This CfP was obtained from [http://www.wikicfp.com/cfp/servlet/event.showcfp?eventid=3845&amp;copyownerid=2048 WikiCFP]
"""]
        else:
            samplesWikiSon = "..."

        return samplesWikiSon

    @classmethod
    def getPropertyLookup(cls, lookupId: str = 'prop') -> dict:
        '''
        get my PropertyLookupList as a map

        Args:
            lookupId(str): name of the lookup id

        Returns:
            dict: my mapping from wiki property names to LoD attribute Names or None if no mapping is defined
        '''
        lookup = None
        if hasattr(cls, 'propertyLookupList'):
            propertyLookupList = getattr(cls, 'propertyLookupList')
            lookup = {prop[lookupId]: prop['name'] for prop in propertyLookupList}
        return lookup

    @classmethod
    def getTemplateParamLookup(cls) -> dict:
        '''
        get my templateParam lookup list as a map
        Returns:
            dict: my mapping from templateParam names to LoD attribute Names or None if no mapping is defined
        '''
        return cls.getPropertyLookup("templateParam")

    @staticmethod
    def postProcessLodRecord(rawEvent:dict, wikiId=None, debug:bool=False):
        '''
        fix the given raw Event
        
        Args:
            rawEvent(dict): the raw event record to fix
            wikiId: wiki id of the records origin
            debug: If True display debug output
        '''
        if wikiId is not None:
            try:
                wikiUser = WikiUser.ofWikiId(wikiId)
                baseUrl=wikiUser.getWikiUrl()
                if 'pageTitle' in rawEvent:
                    pageTitle=rawEvent["pageTitle"]
                    qPageTitle=urllib.parse.quote(pageTitle)
                    url=f"{baseUrl}/index.php?title={qPageTitle}"  #ToDo: Switch to proper page url generation
                    rawEvent['url']=url
            except Exception as e:
                if debug:
                    print(f"WikiUser for {wikiId} not found:  url could not be assigned")
                    print(e)
        rawEvent['eventId']=rawEvent['pageTitle']
        if 'year' in rawEvent:
            yearStr=rawEvent['year']
            rawEvent['yearStr']=yearStr
            year = None
            try:
                year=int(yearStr)
            except Exception as _ne:
                pass
            rawEvent['year']=year
        for dateProp in ['endDate', 'startDate']:
            if dateProp in rawEvent:
                rawDateValue = rawEvent.get(dateProp)
                if isinstance(rawDateValue, str):
                    if rawDateValue:
                        try:
                            dateValue = dateutil.parser.parse(rawEvent.get(dateProp))
                            if dateValue:
                                rawEvent[dateProp] = dateValue
                        except Exception as e:
                            if debug:
                                print(f"{dateProp}: {rawDateValue} → Could not be converted to datetime (event record:{rawEvent})")
                            rawEvent[dateProp] = None
                    else:
                        rawEvent[dateProp] = None
        Ordinal.addParsedOrdinal(rawEvent)


class OREventSeriesManager(EventSeriesManager):
    '''
    i represent a list of EventSeries
    '''

    def __init__(self,
                 sourceConfig:EventDataSourceConfig=None,
                 config:StorageConfig=None,
                 wikiId:str='orclone',
                 via='api',
                 verbose:bool=False,
                 debug=False):
        '''
        construct me
        '''
        self.eventSeries = []
        super(OREventSeriesManager, self).__init__(name="OREventSeries",
                                                   sourceConfig=sourceConfig,
                                                   clazz=OREventSeries,
                                                   primaryKey="pageTitle",
                                                   config=config)
        self.wikiId = wikiId
        self.via = via
        self.debug = debug
        if self.debug:
            self.profile = True
        self.verbose=verbose
        
    def configure(self):
        '''
        configure me
        '''
        if not hasattr(self, "getListOfDicts"):
            if self.via == 'wikiMarkup':
                self.getListOfDicts = self.getLodFromWikiMarkup
            elif self.via == 'backup':
                self.getListOfDicts = self.getLoDfromWikiFileManager
            else:
                self.getListOfDicts = self.getLoDfromWikiUser

    def getLodFromWikiMarkup(self, limit:int=None) -> List[dict]:
        """
        Retrieves the event records from the WikiMarkup of the event pages.

        Args:
            limit(int): limit number of retrieved records. If None (default) retrieve all event records

        Returns:
            LoD List of event records
        """
        lod = OrSMW.getLodFromWikiMarkup(self.wikiId, OREventSeries, limit=limit)
        self.setAllAttr(lod, "source", f"{self.wikiId}-wikiMarkup")
        self.postProcessLodRecords(lod, wikiId=self.wikiId, debug=self.debug)
        return lod

    def getLoDfromWikiUser(self,askExtra:str="", profile: bool=False, limit:int=None) -> List[dict]:
        '''

        Args:
            wikiuser(WikiUser): wikiuser specifiying from which wiki to the records should be queried
            askExtra(str): additional selector for the query e.g. '[[Modification date::>=2022]]'
            profile(bool): Profile the retrieval of the records
            limit(int): limit number of queried records
        '''
        lod = OrSMW.getLodFromWikiApi(self.wikiId, OREventSeries, askExtra=askExtra, limit=limit, profile=profile)
        self.setAllAttr(lod, "source", f"{self.wikiId}")
        self.postProcessLodRecords(lod, wikiId=self.wikiId, debug=self.debug)
        return lod

    def getLoDfromWikiFileManager(self, wikiFileManager: WikiFileManager = None, limit: int = None) -> List[dict]:
        '''
        get my List of Dicts from the given WikiFileManager

        Args:
            wikiFileManager(WikiFileManager): WikiFileManager from which the records should be loaded
            limit(int): limit the amount on loaded records
        '''
        lod = OrSMW.getLodFromWikiFiles(self.wikiId, OREventSeries, wikiFileManager=wikiFileManager, limit=limit)
        self.setAllAttr(lod, "source", f"{self.wikiId}-backup")
        self.postProcessLodRecords(lod, wikiId=self.wikiId, debug=self.debug)
        return lod


    @classmethod
    def getPropertyLookup(cls) -> dict:
        '''
        get my PropertyLookupList as a map

        Returns:
            dict: my mapping from wiki property names to LoD attribute Names or None if no mapping is defined
        '''
        lookup = None
        if hasattr(OREventSeries, 'propertyLookupList'):
            propertyLookupList = OREventSeries.propertyLookupList
            lookup, _duplicates = LOD.getLookup(propertyLookupList, 'prop')
        return lookup

    def fromWikiUser(self, askExtra:str="", limit:int=None, profile:bool=False):
        '''
        read me from a wiki using the given WikiUser configuration

        Args:
            askExtra(string): Extra query selectors that should be included in the query
            limit(int): limit the amount on loaded records
            profile(bool): If true profile the query. Otherwise the query runs without tracking the time
        '''
        lod = self.getLoDfromWikiUser(askExtra, profile=profile, limit=limit)
        self.fromLoD(lod)
        return lod

    def fromWikiFileManager(self, wikiFileManager:WikiFileManager=None, limit:int=None):
        '''
        read me from wiki markup files using the given WikiFileManager

        Args:
            wikiFileManager(WikiFileManager): WikiFileManager to parse the wiki markup files
            limit(int): limit the amount on loaded records
        '''
        lod = self.getLoDfromWikiFileManager(wikiFileManager, limit=limit)
        self.fromLoD(lod)
        return lod


class OREventSeries(EventSeries):
    '''
    an event Series
    '''
    # TODO - this is the legacy templateName - make sure after / during migration
    # this is handled properly
    templateName = "Event series"
    entityName='EventSeries'

    propertyLookupList = [
        {'prop': 'EventSeries acronym', 'name': 'acronym', 'templateParam': 'Acronym'},
        {'prop': 'DblpSeries', 'name': 'dblpSeries', 'templateParam': 'DblpSeries'},
        {'prop': 'Homepage', 'name': 'homepage', 'templateParam': 'Homepage'},
        {'prop': 'Logo', 'name': 'logo', 'templateParam': 'Logo'},
        {'prop': 'Title', 'name': 'title', 'templateParam': 'Title'},
        # TODO enable and handle
        # { 'prop':'Field',      'name': 'subject'},
        {'prop': 'Wikidataid', 'name': 'wikidataId', 'templateParam': 'WikiDataId'},
        {'prop': 'WikiCfpSeries', 'name': 'wikiCfpSeries', 'templateParam': 'WikiCfpSeries'},
        {'prop': 'Period', 'name': 'period', 'templateParam': 'Period'},
        {'prop': 'Unit', 'name': 'unit', 'templateParam': 'Unit'},
        {'prop': 'Has CORE Rank', 'name': 'core2018Rank', 'templateParam': 'has CORE2018 Rank'},
        {'prop': 'GND-ID', 'name': 'gndId', 'templateParam': 'GND-ID'},
        # TODO add more fields according to
        # https://confident.dbis.rwth-aachen.de/or/index.php?title=Template:Event_series&action=edit
    ]

    def __init__(self, wikiFile:WikiFile=None):
        '''
        Constructor
        '''
        super().__init__()

    @classmethod
    def getSamples(cls):
        '''
        Returns a sample LOD of an event Series
        '''
        samplesLOD = [
            {
                'pageTitle': 'AAAI',
                'acronym': 'AAAI',
                'title': 'Conference on Artificial Intelligence',
                'homepage': 'www.aaai.org/Conferences/AAAI/aaai.php',
                'wikidataId': 'Q56682083',
                'dblpSeries': 'aaai',
                'period': 1,
                'unit': 'year',
                'logo':'Aaai-logo.jpg',
                'GND-ID':'309155-7'
            },
            {
                "acronym": "3DUI",
                "creationDate": datetime.fromisoformat("2020-03-17T22:54:10"),
                "dblpSeries": "3dui",
                "lastEditor": "Wolfgang Fahl",
                "modificationDate": datetime.fromisoformat("2021-02-13T06:56:46"),
                "pageTitle": "3DUI",
                "title": "IEEE Symposium on 3D User Interfaces",
                "period": "year",
                "unit": 1,
                "wikidataId": "Q105456162",
                'url': 'https://confident.dbis.rwth-aachen.de/or/index.php?title=3DUI',
                'WikiCfpSeries': 160,
                'core2018Rank':'B',
                'subject': 'User Interfaces',
            },
            ]
        return samplesLOD

    @classmethod
    def getSampleWikiTextList(cls, mode='legacy'):
        '''
        Returns a sample of Event Series in wikison format
        Args:
            mode(str): Default legacy, used to provide the mode dependant on updates and changes to structure of Event series
        '''
        if mode == 'legacy':
            samplesWikiSon = ["""{{Event series
|Acronym=AAAI
|Title=Conference on Artificial Intelligence
|Logo=Aaai-logo.jpg
|has CORE2017 Rank=A*
|Field=Artificial intelligence
|Period=1
|Unit=year
|Homepage=www.aaai.org/Conferences/AAAI/aaai.php
|WikiDataId=Q56682083
|has CORE2018 Rank=A*
|has Bibliography=dblp.uni-trier.de/db/conf/aaai/
|has CORE2014 Rank=A*
|DblpSeries=aaai
}}"""]
        else:
            samplesWikiSon = "..."

        return samplesWikiSon

    @staticmethod
    def postProcessLodRecord(rawEvent: dict, wikiId=None, debug:bool=False):
        '''
        fix the given raw Event

        Args:
            rawEvent(dict): the raw event record to fix
            wikiId: wiki id of the records origin
            debug: If True display debug output
        '''
        if wikiId is not None:
            try:
                wikiUser = WikiUser.ofWikiId(wikiId)
                baseUrl = wikiUser.getWikiUrl()
                if 'pageTitle' in rawEvent:
                    pageTitle = rawEvent["pageTitle"]
                    qPageTitle = urllib.parse.quote(pageTitle)
                    url = f"{baseUrl}/index.php?title={qPageTitle}"   #ToDo: Switch to proper page url generation
                    rawEvent['url'] = url
            except Exception as e:
                if debug:
                    print(f"WikiUser for {wikiId} not found:  url could not be assigned")
                    print(e)
        rawEvent['eventSeriesId'] = rawEvent['pageTitle']
        period = rawEvent.get('period')
        if period and isinstance(period, str):
            if period.isnumeric():
                rawEvent['period'] = int(period)
            else:
                del rawEvent['period']

    @classmethod
    def getPropertyLookup(cls, lookupId: str = 'prop') -> dict:
        '''
        get my PropertyLookupList as a map

        Args:
            lookupId(str): name of the lookup id

        Returns:
            dict: my mapping from wiki property names to LoD attribute Names or None if no mapping is defined
        '''
        lookup = None
        if hasattr(cls, 'propertyLookupList'):
            propertyLookupList = getattr(cls, 'propertyLookupList')
            lookup = {prop[lookupId]: prop['name'] for prop in propertyLookupList}
        return lookup

    @classmethod
    def getTemplateParamLookup(cls) -> dict:
        '''
        get my templateParam lookup list as a map
        Returns:
            dict: my mapping from templateParam names to LoD attribute Names or None if no mapping is defined
        '''
        return cls.getPropertyLookup("templateParam")


class OrSMW:
    """
    Provides different access methods to data in an openresearch wiki
    """

    @classmethod
    def getAskQuery(cls,
                    entityType:Union[Type[OREvent], Type[OREventSeries]],
                    askExtra: str = "",
                    propertyLookupList: List[dict] = None) -> str:
        '''
        get the query that will ask for all records of the given entityType and their properties

        Args:
            entityType(Union[Type[OREvent], Type[OREventSeries]]): entity type
            askExtra(str): any additional SMW ask query constraints
            propertyLookupList:  a list of dicts for propertyLookup

        Return:
            str: the SMW ask query
        '''
        if askExtra is None:
            askExtra = ''
        entityName = entityType.entityName
        selector = "IsA::%s" % entityName
        ask = f"""{{{{#ask:[[{selector}]]{askExtra}
            |mainlabel=pageTitle
            |?Creation date=creationDate
            |?Modification date=modificationDate
            |?Last editor is=lastEditor
            """
        if propertyLookupList is None:
            if hasattr(entityType, 'propertyLookupList'):
                propertyLookupList = getattr(entityType, 'propertyLookupList')
        for propertyLookup in propertyLookupList:
            propName = propertyLookup['prop']
            name = propertyLookup['name']
            ask += "|?%s=%s\n" % (propName, name)
        ask += "}}"
        return ask

    @classmethod
    def getAskQueryPageTitles(cls, entityType:Union[Type[OREvent], Type[OREventSeries]]) -> str:
        """
        get the query that will ask for all pageTitles of the given entityType. e.g. all event pageTitles
        Args:
            entityType(Union[Type[OREvent], Type[OREventSeries]]): entityType

        Returns:
            str: the SMW ask query
        """
        entityName = entityType.entityName
        ask = f"""{{{{#ask:[[IsA::{entityName}]]|mainlabel=pageTitle}}}}"""
        return ask

    @classmethod
    def getLodFromWikiMarkup(cls,
                             wikiId: str,
                             entityType:Union[Type[OREvent], Type[OREventSeries]],
                             limit: int = None,
                             profile:bool=False,
                             showProgress:bool=True) -> List[dict]:
        """
        Retrieves the event records from the WikiMarkup of the event pages.

        Args:
            wikiId: id of the wiki
            entityType: entity to retrieve
            limit(int): limit number of retrieved records. If None (default) retrieve all event records
            profile: If True measure and display the elapsed time
            showProgress: If True display progress messages

        Returns:
            LoD List of entity records
        """
        wikiPush = WikiPush(fromWikiId=wikiId)
        askQuery = cls.getAskQueryPageTitles(entityType=entityType)
        queryDivision = 10 if limit is None or limit >= 1000 else 1
        profiler = Profiler(msg=f"querying of {entityType.entityName} records from WikiMarkup", profile=profile)
        pageTitles = wikiPush.query(askQuery=askQuery, limit=limit, queryDivision=queryDivision)
        wikiPage = WikiPage(wikiId=wikiId, login=False)
        lod = []
        total = len(pageTitles)
        for i, pageTitle in enumerate(pageTitles):
            if showProgress:
                print(f"({i+1}/{total}) Extracting {entityType.templateName} record from {pageTitle} ...", end=' ')
            try:
                record = wikiPage.getWikiSonFromPage(pageTitle, entityType.templateName, includeWikiMarkup=True)
                record = cls.normalizeProperties(record, entityType, force=False, includeProps=['pageTitle', 'wikiMarkup'])
                record['pageTitle'] = pageTitle
                lod.append(record)
                if showProgress:
                    print('✅')
            except Exception as e:
                if showProgress:
                    print('❌')
        profiler.time()
        return lod

    @classmethod
    def getLodFromWikiApi(cls,
                          wikiId:str,
                          entityType:Union[Type[OREvent], Type[OREventSeries]],
                          askExtra:str=None,
                          limit:int=None,
                          profile:bool=False) -> List[dict]:
        """
        Retrieves the event records from the SMW api by querying it for all entity properties.

        Args:
            wikiId: id of the wiki
            entityType: entity to retrieve
            askExtra: extra ask query selector to include in the query
            limit: limit number of retrieved records. If None (default) retrieve all event records
            profile: If True measure and display the elapsed time

        Returns:
            LoD: List of entity records
        """
        wikiPush = WikiPush(fromWikiId=wikiId)
        askQuery = cls.getAskQuery(entityType=entityType, askExtra=askExtra)
        queryDivision = 10 if limit is None or limit >= 1000 else 1
        profiler = Profiler(msg=f"querying of {entityType.entityName} records over SMW api", profile=profile)
        records = wikiPush.formatQueryResult(askQuery, limit=limit, queryDivision=queryDivision)
        profiler.time()
        return records

    @classmethod
    def getLodFromWikiFiles(cls,
                          wikiId: str,
                          entityType: Union[Type[OREvent], Type[OREventSeries]],
                          wikiFileManager: WikiFileManager = None,
                          limit: int = None,
                          profile: bool = False) -> List[dict]:
        """
        Retrieves the event records from the local wikiFiles. The entity records are parsed from the WikiMarkup in the
        WikiFiles.

        Args:
            wikiId: id of the wiki
            entityType: entity to retrieve
            wikiFileManager: wikiFileManager to use. If None the default wikibackup location (~/.or/wikibackup/<wikiId>) will be used
            limit: limit number of retrieved records. If None (default) retrieve all event records
            profile: If True measure and display the elapsed time

        Returns:
            LoD: List of entity records
        """
        profiler = Profiler(msg=f"querying of {entityType.entityName} records over SMW api", profile=profile)
        if wikiFileManager is None:
            wikiTextPath = f"{Path.home()}/.or/wikibackup/{wikiId}"
            wikiFileManager = WikiFileManager(sourceWikiId=wikiId, wikiTextPath=wikiTextPath, login=False)
        wikiFileDict = wikiFileManager.getAllWikiFiles()
        lod = wikiFileManager.convertWikiFilesToLOD(wikiFiles=list(wikiFileDict.values()),
                                                    templateName=entityType.templateName,
                                                    limit=limit)
        for i, record in enumerate(lod):
            pageTitle = record.get("pageTitle")
            record = cls.normalizeProperties(record, entityType, force=False)
            record['pageTitle'] = pageTitle
            wikiFile = wikiFileDict.get(pageTitle)
            if isinstance(wikiFile, WikiFile):
                # ToDo: Decide whether the wikiMarkup is still required since it can now an event record can directly be queried from the wiki, sync problems should be neglectable
                record["wikiMarkup"] = wikiFile.wikiText
            lod[i]=record
        profiler.time()
        return lod

    @staticmethod
    def normalizeProperties(record: dict,
                            entity: Union[Type[OREvent], Type[OREventSeries]],
                            reverse: bool = False,
                            force: bool = False,
                            includeProps:list=None,
                            debug: bool = False) -> dict:
        """
        update the keys of the given record. If reverse is False normalize the given keys with the property map provided
        by the given entity. If reverse is True denormalize the keys back to the template params.

        Normalize: Acronym (template param) -> acronym (normalized param)
        Denormalize: acronym (normalized param) -> Acronym (template param)

        Args:
            record(dict): record to normalize
            entity: entity containing the template property mapping
            reverse(bool): If False normalize. Otherwise, denormalize back to template property names
            force(bool): If True include properties that are not present in the getTemplateParamLookup() of the given
                         entity. Otherwise, exclude these properties and show a warning.
            includeProps(list): List of properties that should be included even if not in getTemplateParamLookup()
            debug(bool): If True show debug messages.

        Returns:
            dict
        """
        if includeProps is None:
            includeProps = []
        propMap = entity.getTemplateParamLookup()
        if reverse:
            propMap = {v: k for k, v in propMap.items()}
        res = {}
        for key, value in record.items():
            if key in propMap:
                res[propMap[key]] = value
            else:
                if force or key in includeProps:
                    res[key] = value
                else:
                    msg = f"Property '{key}' will be excluded (not present in the getTemplateParamLookup() of {entity.__name__}). To include this property use force=True"
                    if debug:
                        print(msg)
        return res


'''
Created on 2022-04-03

@author: wf
'''
import re
from typing import List, Tuple, Dict

from lodstorage.lod import LOD

from ptp.signature import OrdinalCategory


class EventSeriesCompletion(object):
    '''
    complete a series of events from different datasources 
    '''
    tokenizer=None
    
    def __init__(self):
        '''
        Constructor
        '''            
    
    
    @classmethod
    def getCompletedBlankSeries(cls, lod: List[dict], debug: bool = False) -> list:
        """
        gets all blank entries of a series from the different records that are given.
        It is expected that each lod comes from a  different datasource
        Args:
            lods: list of event records

        Returns:
            List of completed year ordinal pairs, or empty list if given lod can not be completed
        """
        yearOrdinalPair = {(r.get('year'), r.get("ordinal")) for r in lod }
        completeYearOrdinalPair = list({(year, ordinal) for year, ordinal in yearOrdinalPair if year is not None and ordinal is not None})
        completeYearOrdinalPair.sort(key=lambda r: r[0] if r[0] is not None else 0)
        cleanedYOP = [completeYearOrdinalPair[0]]
        ordinal = completeYearOrdinalPair[0][1]
        for year, ord in completeYearOrdinalPair[1:]:
            if ordinal < ord:
                cleanedYOP.append((year, ord))
                ordinal = ord
        completeYearOrdinalPair = cleanedYOP
        ordByYear = {}
        for year, ord in completeYearOrdinalPair:
            if year in ordByYear:
                ordByYear[year].append(ord)
            else:
                ordByYear[year] = [ord]
        if debug:
            yearsWithKnownOrd = {year for year, ordinal in completeYearOrdinalPair}
            ordinalWithKnownYear = {ordinal for year, ordinal in completeYearOrdinalPair}
            yearWithUnknownOrdinal = {year for year, ordinal in yearOrdinalPair if year not in yearsWithKnownOrd and year is not None}
            ordinalWithUnkownYear = {ordinal for year, ordinal in yearOrdinalPair if ordinal not in ordinalWithKnownYear and ordinal is not None}
            print("Years without ordinal", yearWithUnknownOrdinal)
            print("Ordinals with unknown Year", ordinalWithUnkownYear)
            print("Year ordinal pairs: ", completeYearOrdinalPair)
            print("Ordinals by year: ", ordByYear)
        hasMultipleOrdinalsForAYear = any([len(ords)>1 for ords in ordByYear.values()])
        if hasMultipleOrdinalsForAYear:
            if debug:
                print("Multiple ordinals per year → can not complete series")
            return []
        else:
            frequency = cls.getFrequency(completeYearOrdinalPair)
            if frequency:
                # series is frequency consistent
                lastKnownEvent = completeYearOrdinalPair[-1]
                inception = lastKnownEvent[0] - (frequency * lastKnownEvent[1])
                return [(inception+(i*frequency), i) for i in range(1,lastKnownEvent[1]+1)]
        return []

    @classmethod
    def getFrequency(cls, yearOrdinalPairs:List[Tuple[int, int]], debug:bool=False) -> int:
        """
        Determines the Frequency of the given year ordinal pairs.
        Only returns the frequency if it is consistent over the complete series.

        Args:
            yearOrdinalPairs (list): list of year ordinal pairs for which the frequency should be determined

        Returns:
            Returns the frequency or 0 if inconsistent or if the frequency could not be determined
        """
        if len(yearOrdinalPairs) <= 1:
            return 0
        yearOrdinalPairs.sort(key=lambda r: r[0] if r[0] is not None else 0)
        ordMonotonicallyIncreasing = all([ord < yearOrdinalPairs[i+1][1] for i, (year, ord) in enumerate(yearOrdinalPairs[:-1])])
        if not ordMonotonicallyIncreasing:
            return 0
        firstKnownEvent = yearOrdinalPairs[0]
        lastKnownEvent = yearOrdinalPairs[-1]
        if debug:
            print("First:", firstKnownEvent, "Last:", lastKnownEvent, "#YearOrdPairs:", len(yearOrdinalPairs))#
        timeDelta = lastKnownEvent[0] - firstKnownEvent[0]
        ordinalDelta = lastKnownEvent[1] - firstKnownEvent[1]
        frequency = timeDelta / ordinalDelta
        yearsBetweenEvents = {(year - yearOrdinalPairs[i][0])/(ord - yearOrdinalPairs[i][1]) for i, (year, ord) in enumerate(yearOrdinalPairs[1:])}
        if frequency.is_integer() and len(yearsBetweenEvents) == 1 and int(frequency) in yearsBetweenEvents:
            return int(frequency)
        return 0


    @staticmethod
    def filterDuplicatesByTitle(lod: List[dict], debug: bool = False) -> List[dict]:
        """
        tries to filter out duplicate event records
        currently only for tibkat and dblp records
        Args:
            lod(List[dict]): List of event records from one sources
            debug(bool): if True show debug messages

        Returns:
            List[dict]
        """
        byYear = LOD.getLookup(lod, "year", withDuplicates=True)
        res = []
        for year, records in byYear.items():
            if len(records) > 1:
                recordsRanked = [(TitleScore.getScore(record.get("title")), record) for record in records]
                recordsRanked.sort(key=lambda pair:pair[0], reverse=True)
                if debug:
                    print(f"==={year}===")
                    for score, record in recordsRanked:
                        print(f"* {score} - '{record.get('title')}'")
                maxScore = recordsRanked[0][0]
                records = [record for score, record in recordsRanked if score == maxScore]
                res.extend(records)
            else:
                res.append(records[0])
        return res


class TitleScore:
    """
    Calculates a rating for an event title
    A higher score represents a better fit of the record as first/correct record for an event with multiple records
    """
    VOLUME_LOOKUP = OrdinalCategory()

    @classmethod
    def getScore(cls, title:str)-> float:
        """
        calc title score
        Args:
            title: event title

        Returns:
            float - score
        """
        if title is None:
            return 0.0
        score = cls._hasProceeding(title) + cls._hasEventType(title)
        volume = cls._getVolume(title)
        if volume:
            score += 1/volume
        score += (1/max(1, cls._countSpecialCharacters(title))) / 2
        return score

    @classmethod
    def _getVolume(cls, title:str) -> int:
        #ToDo: Migrate to VolumeCategory parsing
        volumePattern = r"\[?([Vv]ol\.|[Vv]olume|[Pp]t\.|[Pp]art)\]?( |)(?P<volumeNumber>\d{1,2}|[A-H]|(IX|IV|V?I{0,3}))"
        match = re.search(volumePattern, title)
        volume = None
        if match is not None:
            volume = cls.VOLUME_LOOKUP.lookup(match.group("volumeNumber") + '.')  # adding dot as workaround to be able to use the ordinal parser
        return volume

    @classmethod
    def _hasProceeding(cls, title:str):
        return cls._titleContains("proceeding", title)

    @classmethod
    def _hasEventType(cls, title:str):
        return cls._titleContains("conference", title)

    @classmethod
    def _titleContains(cls, value:str, title:str):
        return 1.0 if title is not None and value in title.lower() else 0.0

    @classmethod
    def _countSpecialCharacters(cls, title:str):
        res = sum([1 for char in title if not(char.isalpha() or char.isdigit() or ' ')])
        return res



'''
Created on 2021-07-30

@author: wf
'''
from corpus.event import EventStorage
from corpus.eventcorpus import EventCorpus, EventDataSource
from corpus.datasources.acm import ACM
from corpus.datasources.confref import Confref
from corpus.datasources.crossref import Crossref
from corpus.datasources.dblp import Dblp
from corpus.datasources.gnd import GND
from corpus.datasources.tibkat import Tibkat
from corpus.datasources.wikidata import Wikidata
from corpus.datasources.openresearch import OR
from corpus.datasources.wikicfp import WikiCfp

from lodstorage.lod import LOD
from wikibot.wikiuser import WikiUser
from wikifile.wikiFileManager import WikiFileManager

import os
import re
from os import path
import sqlite3
import sys

from argparse import ArgumentParser
from argparse import RawDescriptionHelpFormatter


class CorpusLookupConfigure:
    '''
    Configuration of the Corpus Lookup
    '''
    @staticmethod
    def getWikiTextPath(wikiId:str):
        ''' 
        get the WikiText (Backup) path for the given wikiId
        
        Args:
            wikiId(str): the wikiId (e.g. "or", "orclone"
            
        Return:
            the path to the backup files as created by the wikibackup script
        '''
        home = path.expanduser("~")
        wikiTextPath = f"{home}/.or/wikibackup/{wikiId}"
        return wikiTextPath
            
    @staticmethod
    def configureCorpusLookup(lookup,debug=False):
        '''
        callback to configure the corpus lookup
        '''
        print("configureCorpusLookup callback called")
        pass

class CorpusLookup(object):
    '''
    search and lookup for different EventCorpora
    '''
    lookupIds=["confref","crossref","dblp","gnd","tibkat","wikidata","wikicfp","or","or-backup","orclone","orclone-backup"]
    

    def __init__(self,lookupIds:list=None,
                 configure:callable=None,debug=False):
        '''
        Constructor
        
        Args:
            lookupIds(list): the list of lookupIds to addDataSources for
            configure(callable): Callback to configure the corpus lookup
        '''
        self.debug=debug
        self.configure=configure
        self.eventCorpus=EventCorpus()
        if lookupIds is None:
            lookupIds=CorpusLookup.lookupIds
        if "acm" in lookupIds:
            self.eventCorpus.addDataSource(ACM())
        if "confref" in lookupIds:
            self.eventCorpus.addDataSource(Confref())
        if "crossref" in lookupIds:
            self.eventCorpus.addDataSource(Crossref())
        if "dblp" in lookupIds:
            self.eventCorpus.addDataSource(Dblp())
        if "gnd" in lookupIds:
            self.eventCorpus.addDataSource(GND())
        if "tibkat" in lookupIds:
            self.eventCorpus.addDataSource(Tibkat())
        if "wikidata" in lookupIds: 
            self.eventCorpus.addDataSource(Wikidata())
        if "wikicfp" in lookupIds:    
            self.eventCorpus.addDataSource(WikiCfp())
        if "or" in lookupIds:    
            self.eventCorpus.addDataSource(OR(wikiId="or",via="api"))
        if "or-backup" in lookupIds:    
            self.eventCorpus.addDataSource(OR(wikiId="or",via="wikiMarkup"))
        if "orclone" in lookupIds:    
            self.eventCorpus.addDataSource(OR(wikiId="orclone",via="api"))
        if "orclone-backup" in lookupIds:    
            self.eventCorpus.addDataSource(OR(wikiId="orclone",via="wikiMarkup"))
        
    def getDataSource(self,lookupId:str)->EventDataSource:
        '''
        get the data source by the given lookupId
        
        Args:
            lookupId(str): the lookupId of the data source to get
            
        Return:
            EventDataSource: the data source

        '''
        eventDataSource=None
        if lookupId in self.eventCorpus.eventDataSources:
            eventDataSource=self.eventCorpus.eventDataSources[lookupId]
        return eventDataSource

    def getDataSource4TableName(self,tableName:str)->EventDataSource:
        '''
        get the data source by the given tableName

        Args:
            tableName(str): a tableName of the data source to get

        Return:
            EventDataSource: the data source

        '''
        for eventDataSource in self.eventCorpus.eventDataSources.values():
            if eventDataSource.eventManager.tableName==tableName:
                return eventDataSource
            if eventDataSource.eventSeriesManager.tableName==tableName:
                return eventDataSource
        return None


    def load(self,forceUpdate:bool=False,showProgress:bool=False,withCreateViews=True):
        '''
        load the event corpora
        
        Args:
            forceUpdate(bool): if True the data should be fetched from the source instead of the cache
            showProgress(bool): if True the progress of the loading should be shown
            withCreateViews(bool): if True recreate the common views
        '''
        if self.configure:
            self.configure(self)
        self.eventCorpus.loadAll(forceUpdate=forceUpdate,showProgress=showProgress)
        if withCreateViews:
            EventStorage.createViews(exclude=EventStorage.viewTableExcludes)
    
    def getDataSourceInfos(self,withInstanceCount:bool=True):
        '''
        get the dataSource Infos
        '''
        infos=[]
        for dataSourceName,dataSource in self.eventCorpus.eventDataSources.items():
            info={
                "source":dataSourceName,
                "title": dataSource.sourceConfig.title,
                "url":dataSource.sourceConfig.url,
                "lookupId": dataSource.sourceConfig.lookupId,
                "tableSuffix": dataSource.sourceConfig.tableSuffix
            }
            if withInstanceCount:
                em=dataSource.eventManager
                esm=dataSource.eventSeriesManager
                for title,manager in [("event",em),("series",esm)]:
                    query=f"SELECT count(*) as count from {manager.tableName}"
                    lod=self.getLod4Query(query)
                    record=lod[0]
                    info[title]=record["count"]
            infos.append(info)
        return infos
            
    def getLod4Query(self,query:str,params=None):
        '''
        Args:
            query: the query to run
            params(tuple): the query params, if any
        Return:
            list: the list of dicts for the query
        '''
        sqlDB=EventStorage.getSqlDB()
        listOfDicts=sqlDB.query(query,params)
        return listOfDicts
    
    def getMultiQueryVariable(self,multiquery:str,lenient:bool=False):
        '''
        get the variable being used in a multiquery
        
        Args:
            multiquery(str): the multiquery containing a {variable}
            lenient(bool): if True allow to return a None value otherwise raise an Exception if no variable was found
            
        Returns:
            str: variable
            
        Raises:
            Exception: if lenient is False and no variable was found
            
        '''
        var=None
        regEx=r"\{(.*)\}"
        varMatch=re.search(regEx, multiquery)
        # which variable to we want to replace?
        if varMatch:
            var=varMatch.group(1)
        else:
            if not lenient:
                raise Exception("need a variable for the tableName to be queried in {viewName} notation")   
        return var
    
    def getDictOfLod4MultiQuery(self,multiquery:str,idQuery:str=None,omitFailed:bool=True)->dict:
        '''
        Args:
            multiquery(str): the multi query containing a variable
            idQuery(str): optional query to get lists of ids for selection
            omitFaild(bool): if True omit failed queries if False raise Exception on failure
            
        Return:
            dict: the dict of list of dicts for the queries derived
            from the multi query
            
        Raises:
            Exception: if omitFailed is False and an error occured for a query
        '''
        dataSourceInfos=self.getDataSourceInfos(withInstanceCount=False)
        dataSourcesByTableSuffix,_dup=LOD.getLookup(dataSourceInfos, "tableSuffix")
        viewName=self.getMultiQueryVariable(multiquery)
        variable="{%s}" % viewName
        dictOfLod={}
        # if an idquery is given create a dictionary of ids pers source
        if idQuery:
            idLod=self.getLod4Query(idQuery)
            idDict={}
            idColumn=f"{viewName}Id"
            for idRecord in idLod:
                source=idRecord["source"]
                recordId=f"'{idRecord[idColumn]}'"
                if source in idDict:
                    idDict[source].append(recordId)
                else:
                    idDict[source]=[recordId]
        # get the tables to potentially query 
        tableList=EventStorage.getViewTableList(viewName, exclude=EventStorage.viewTableExcludes)
        # loop over all relevant tables
        for table in tableList:
            tableName=table["name"]
            # create the base query per table by replacing the viewname for the base table
            queryPrefix=multiquery.replace(variable,tableName)
            tableSuffix=tableName.replace(f"{viewName}_","")
            # do we have a data source for the table?
            if tableSuffix in dataSourcesByTableSuffix:
                # are we going to actually query?
                queryDataSource=True
                whereClause=""
                # get the dataSource
                dataSourceName=dataSourcesByTableSuffix[tableSuffix]["lookupId"]
                # shall we select only certain ids?
                if idQuery:
                    # do we have an idList?
                    if dataSourceName in idDict: 
                        idList=",".join(idDict[dataSourceName])
                        whereClause=f" WHERE {idColumn} in ({idList})"
                    else:
                        # no idList - do not query at all
                        queryDataSource=False
                # if we want to query
                if queryDataSource:
                    # query the table
                    query=f"{queryPrefix}{whereClause}"
                    try:
                        # get the list of dicts
                        lod=self.getLod4Query(query)
                        # put the result into the dict for the given datasource
                        dictOfLod[dataSourceName]=lod
                    except sqlite3.OperationalError as ex:
                        if not omitFailed:
                            raise ex
        return dictOfLod
        
        
__version__ = "0.0.27"
__date__ = '2020-06-22'
__updated__ = '2022-01-03'

DEBUG = 1

    
def main(argv=None): # IGNORE:C0111
    '''main program.'''

    if argv is None:
        argv = sys.argv
    else:
        sys.argv.extend(argv)    
        
    program_name = os.path.basename(sys.argv[0])
    program_version = "v%s" % __version__
    program_build_date = str(__updated__)
    program_version_message = '%%(prog)s %s (%s)' % (program_version, program_build_date)
    program_shortdesc = "Scientific Event Corpus and Lookup"
    user_name="Wolfgang Fahl"
    program_license = '''%s

  Created by %s on %s.
  Copyright 2020-2022 Wolfgang Fahl. All rights reserved.

  Licensed under the Apache License 2.0
  http://www.apache.org/licenses/LICENSE-2.0

  Distributed on an "AS IS" basis without warranties
  or conditions of any kind, either express or implied.

USAGE
''' % (program_shortdesc, user_name,str(__date__))

    try:
        # Setup argument parser
        datasourcesDefault=",".join(CorpusLookup.lookupIds)
        parser = ArgumentParser(description=program_license, formatter_class=RawDescriptionHelpFormatter)
        parser.add_argument("-d", "--debug", dest="debug", action="store_true", help="show debug info")
        parser.add_argument("-q", "--query",help="run the given query")
        parser.add_argument('-e', '--endpoint', default=Wikidata.endpoint, help="SPARQL endpoint to use for wikidata queries")
        parser.add_argument('-v', '--version', action='version', version=program_version_message)
        parser.add_argument("-u", "--uml", dest="uml", action="store_true", help="output plantuml diagram markup")
        parser.add_argument("-f", "--force",dest="forceUpdate",action="store_true",help="force Update - may take quite a time")
        parser.add_argument("--datasources",help=", delimited list of datasource lookup ids",default=datasourcesDefault)
        
        # Process arguments
        args = parser.parse_args()   
        Wikidata.endpoint=args.endpoint
        lookupIds=args.datasources.split(",")
        lookup=CorpusLookup(debug=args.debug,lookupIds=lookupIds,configure=CorpusLookupConfigure.configureCorpusLookup)
        lookup.load(forceUpdate=args.forceUpdate)
        if args.uml:
            for baseEntity in ["Event","EventSeries"]:
                plantUml=lookup.asPlantUml(baseEntity)
                print(plantUml)
        if args.query:
            lookup.query(args.query)

        
    except KeyboardInterrupt:
        ### handle keyboard interrupt ###
        return 1
    except Exception as e:
        if DEBUG:
            raise(e)
        indent = len(program_name) * " "
        sys.stderr.write(program_name + ": " + repr(e) + "\n")
        sys.stderr.write(indent + "  for help use --help")
        return 2         
        
if __name__ == "__main__":
    if DEBUG:
        sys.argv.append("-d")
    sys.exit(main())



'''
Created on 2021-08-07

@author: wf
'''
from corpus.quality.rating import EntityRating

class EventRating(EntityRating):
    '''
    a rating for an event
    '''
    
    def __init__(self,event):
        super().__init__(self,event,event.eventId,event.source,"Event")
        
class EventSeriesRating(EntityRating):
    '''
    a rating for an event
    '''
    
    def __init__(self,eventSeries):
        super().__init__(self,eventSeries,eventSeries.seriesId,eventSeries.source,"EventSeries")
        


'''
Created on 2021-12-27

@author: wf
'''
from OSMPythonTools.cachingStrategy import CachingStrategy, JSON
from OSMPythonTools.nominatim import Nominatim
from geopy.geocoders import Nominatim as GeoNominatim

from pathlib import Path
import os
import logging

class NominatimWrapper(object):
    '''
    Nominatim Wrapper to hide technical details of Nominatim interface
    '''

    def __init__(self, cacheDir:str=None,user_agent:str="ConferenceCorpus"):
        '''
        Constructor

            create a nominatim instance for the given cacheDir - if cacheDir is None use ~/.nominatim as cachedir
        
        Args:
            cacheDir(str): the path to the cache directory to be use by Noninatims JSON caching  Strategy
            user_agent(str): the user_agent to use for the geolocator
            
        '''
        if cacheDir is None:
            home = str(Path.home())
            cacheDir=f"{home}/.nominatim"     
        self.cacheDir=cacheDir
        if not os.path.exists(self.cacheDir):
            os.makedirs(cacheDir)
        logging.getLogger('OSMPythonTools').setLevel(logging.ERROR)     
        CachingStrategy.use(JSON, cacheDir=cacheDir)
        self.nominatim = Nominatim()  
        self.geolocator = GeoNominatim(user_agent=user_agent)

 
    def lookupWikiDataId(self,locationText:str):
        '''
        lookup the Wikidata Identifier for the given locationText (if any)
        
        Args:
            locationText(str): the location text to search for
            
        Return:
            the wikidata Q identifier most fitting the given location text
            
        '''
        wikidataId=None
        nresult=self.nominatim.query(locationText,params={"extratags":"1"})
        nlod=nresult._json
        if len(nlod)>0:
            nrecord=nlod[0]
            if "extratags" in nrecord:
                extratags=nrecord["extratags"]
                if "wikidata" in extratags:
                    wikidataId=extratags["wikidata"]
        return wikidataId
  
        
        


'''
Created on 19.05.2022

@author: wf
'''
import datetime
class Figure():
    '''
    a figure 
    '''
    def __init__(self,title,caption,figLabel,sqlQuery,fileNames,px=600,scale=None):
        '''
        constructore
        '''
        self.title=title
        self.caption=caption
        self.sqlQuery=sqlQuery
        self.figLabel=figLabel
        self.fileNames=fileNames
        self.px=px
        self.scale=scale
        self.scalemarkup=f"[scale={scale}]" if scale else ""
        
    def asWikiMarkup(self):
        '''
        create mediawiki markup to show the given outputFilename and sqlQuery
        '''
        markup=f"""== {self.title} =="""
        if self.sqlQuery:
            markup+=f"""
=== sql query ===
<source lang='sql'>
{self.sqlQuery}
</source>"""
        markup+=f"""
=== {self.caption} ===
"""
        for fileName in self.fileNames:
            markup+=f"""[[File:{fileName}|{self.px}px]]"""
        return markup 
    
    def asLatex(self,index,cols):
        '''
        get the latex Code for the given figure
        
        Args:
            index(int): the index of this figure
            cols(int): total number of cols
        '''
        latex=""
        if cols==1 or index==0:
            latex="\\begin{figure}"
            latex+="\n"
        fileCount=len(self.fileNames)
        for fileName in self.fileNames:
            latex+=f"""    %figure {self.figLabel}:{self.caption}
    {{\\includegraphics{self.scalemarkup}[width=\\linewidth/{fileCount}/{cols}]{{{self.latexFolder}{fileName}}}}}%
    \hfill%"""
        if cols==1:
            latex+=Figure.footer(self.caption,self.figLabel)
        else:
            if (index+1)%cols==0:
                latex+="\n    \\\\[\smallskipamount]"
        return latex 
    
    @classmethod
    def footer(cls,caption,figLabel):
        latex="""
    \caption{%s}
    \label{fig:%s}
\end{figure}""" % (caption,figLabel)
        return latex
    
class FigureList():
    '''
    Figure List
    '''
    maxFigureListId=0
    
    def __init__(self,caption,latexFolder="results/",figureListLabel=None,cols=1):
        '''
        constructor
        '''
        self.caption=caption
        self.clear()
        self.latexFolder=latexFolder
        self.isodate=datetime.datetime.now().isoformat()
        if figureListLabel is None:
            FigureList.maxFigureListId+=1
            figureListLabel=f"figureList{FigureList.maxFigureListId}"
        self.figureListLabel=figureListLabel
        self.cols=cols
    
    def clear(self):
        '''
        clear my figures list
        '''
        self.figures=[]      
        
    def add(self,figure:Figure):
        '''
        add the given figure
        
        figure(Figure): the figure to add
        '''
        figure.latexFolder=self.latexFolder
        self.figures.append(figure)
        
    def latexComment(self,preamble:bool=True):
        '''
        create a latex comment
        
        Args:
            preamble(bool): if True generate a preamble comment otherwise a footer comment
        '''
        if preamble:
            text=f"""% >>>figureList {self.figureListLabel} generated by {__file__} on {self.isodate}"""
        else:
            text=f"%<<<figureList {self.figureListLabel}: end of generated latex code"
        return text
    
    def printAllMarkups(self):
        '''
        print all Markus
        '''
        for figure in self.figures:
            print(figure.asWikiMarkup())
        # wrap 
        print(self.latexComment(True))
        for index,figure in enumerate(self.figures):
            print(figure.asLatex(index,self.cols))
        if self.cols>=1:
            print(Figure.footer(self.caption, self.figureListLabel))
        print(self.latexComment(False))


'''
Created on 2020-07-11

@author: wf
'''
import html
import re
import os
import json
from corpus.event import Event,EventSeries,EventManager,EventSeriesManager
from corpus.eventcorpus import EventDataSourceConfig,EventDataSource
from lodstorage.storageconfig import StorageConfig
from lodstorage.sql import SQLDB
from ptp.ordinal import Ordinal

class Confref(EventDataSource):
    '''
    ConfRef platform
    '''
    sourceConfig=EventDataSourceConfig(lookupId="confref",name="confref.org",url="http://portal.confref.org",title="ConfRef",tableSuffix="confref",locationAttribute="location")
    
    def __init__(self):
        '''
        construct me 
        '''
        super().__init__(ConfrefEventManager(),ConfrefEventSeriesManager(),Confref.sourceConfig)

    @staticmethod
    def htmlUnEscapeDict(htmlDict:dict):
        '''
        perform html unescaping on the given dict
        
        Args:
            htmlDict(dict): the dictionary to unescape
        '''
        for key in htmlDict:
            value=htmlDict[key]
            if value is not None and type(value) is str:
                value=html.unescape(value)
                htmlDict[key]=value
                
class ConfrefEvent(Event):
    '''
    a scientific event derived from Confref
    '''
    
    @staticmethod
    def postProcessLodRecord(rawEvent:dict):
        '''
        fix the given raw Event
        
        Args:
            rawEvent(dict): the raw event record to fix
        '''
        Confref.htmlUnEscapeDict(rawEvent)
        
        eventId=rawEvent.pop('id')
        # rename number to ordinal
        if 'number' in rawEvent:
            rawEvent['ordinal']=rawEvent.pop('number')
        # handle area and confSeries dicts
        _area=rawEvent.pop('area')
        if isinstance(_area,dict):
            Confref.htmlUnEscapeDict(_area)
            rawEvent["area"]=_area["value"]
            pass
        _confSeries=rawEvent.pop('confSeries')
        if isinstance(_confSeries,dict):
            # dict: 
            #   {'id': 'btw', 
            #    'issn': None, 
            #    'eissn': None, 
            #    'dblpId': 'https://dblp.org/db/conf/btw/', 
            #    'name': 'Datenbanksysteme für Business, Technologie und Web Datenbanksysteme in Büro, Technik und Wissenschaft', 
            #    'description': None
            #  }
            #
            Confref.htmlUnEscapeDict(_confSeries)
            dblpSeriesId=_confSeries["dblpId"]
            if dblpSeriesId is not None:
                m=re.match("https://dblp.org/db/(.*)/",dblpSeriesId) 
                if m:
                    dblpSeriesId=m.group(1)
            rawEvent['dblpSeriesId']=dblpSeriesId
            rawEvent['seriesId']=_confSeries["id"]
            rawEvent['seriesTitle']=_confSeries["name"]   
            rawEvent['seriesIssn']=_confSeries["issn"]
            rawEvent['seriesEissn']=_confSeries["eissn"]
        rawEvent['eventId']=eventId
        rawEvent['url']=f'http://portal.confref.org/list/{eventId}'        
        rawEvent['title']=rawEvent.pop('name')
        rawEvent["source"]="confref"
        location=None
        if "city" in rawEvent and "country" in rawEvent:
            location=f"""{rawEvent["city"]},{rawEvent["country"]}"""
        rawEvent["location"]=location
        Ordinal.addParsedOrdinal(rawEvent)
        pass
    
    
    def fromDict(self,rawEvent:dict):
        '''
        get me from the given dict
        '''
        super().fromDict(rawEvent)
    
class ConfrefEventSeries(Event):
    '''
    a scientific event series derived from Confref
    '''
        
class ConfrefEventManager(EventManager):
    '''
    Crossref event manager
    '''
        
    def __init__(self, config: StorageConfig = None):
        '''
        Constructor
        '''
        super().__init__(name="ConfrefEvents", sourceConfig=Confref.sourceConfig, clazz=ConfrefEvent, config=config)
        
    def configure(self):
        '''
        configure me
        '''
        # nothing to do - there is a get ListOfDicts below
    
    def getListOfDicts(self):
        '''
        get my content from the json file
        '''
        cachePath=self.config.getCachePath()
        jsondir=f"{cachePath}/confref"
        if not os.path.exists(jsondir):
                os.makedirs(jsondir)
        self.jsonFilePath=f"{jsondir}/confref-conferences.json"
        with open(self.jsonFilePath) as jsonFile:
            rawEvents=json.load(jsonFile)
        lod=[]
        for rawEvent in rawEvents:
            lod.append(rawEvent)
        self.postProcessLodRecords(lod)
        return lod
        
class ConfrefEventSeriesManager(EventSeriesManager):
    '''
    Confref event series handling
    '''
    
    def __init__(self,config:StorageConfig=None):
        '''
        Constructor
        '''
        super().__init__(name="ConfrefEventSeries", sourceConfig=Confref.sourceConfig,clazz=ConfrefEventSeries,config=config)


    def configure(self):
        '''
        configure me
        '''
        # nothing to do getListOfDicts is defined
        
    def getListOfDicts(self):
        '''
        get my data
        '''
        query="""select dblpSeriesId as eventSeriesId,acronym,seriesTitle as title,count(*) as count,min(year) as minYear,max(year) as maxYear
from event_confref
where dblpSeriesId is not Null
group by dblpSeriesId"""
        sqlDB=SQLDB(self.getCacheFile())
        listOfDicts=sqlDB.query(query)
        self.setAllAttr(listOfDicts, "source", "confref")
        self.postProcessLodRecords(listOfDicts)
        return listOfDicts
 


'''
Created on 2020-07-05

@author: wf
'''
from ptp.ordinal import Ordinal
from corpus.event import Event, EventSeries, EventManager, EventSeriesManager
from lodstorage.storageconfig import StorageConfig
from lodstorage.lod import LOD
import json
import re
import glob
import os
import time
from datetime import datetime
from corpus.eventcorpus import EventDataSource, EventDataSourceConfig
import habanero


class Crossref(EventDataSource):
    '''
        Access to Crossref's search api see https://github.com/CrossRef/rest-api-doc
    '''
    sourceConfig = EventDataSourceConfig(lookupId="crossref", name="crossref.org", url="https://www.crossref.org/", title="CrossRef", tableSuffix="crossref",locationAttribute="location")
    cr = habanero.Crossref()  
       
    def __init__(self):
        '''
        constructor
        '''
        super().__init__(CrossrefEventManager(), CrossrefEventSeriesManager(), Crossref.sourceConfig)
     
    @classmethod 
    def doiMetaData(cls, doi):
        ''' get the meta data for the given doi '''
        metadata = None
        response = cls.cr.works([doi])
        if 'status' in response and 'message' in response and response['status'] == 'ok':
            metadata = response['message']
        return metadata
    
    @staticmethod    
    def fixEncodings(eventInfo:dict, debug:bool=False): 
        '''
        fix the encoding of the dict entries of the given eventInfo
        
        Args:
            eventInfo(dict): the record to fix
            debug(bool): show debug information if True
        '''
        for keyValue in eventInfo.items():
            key, value = keyValue
            oldvalue = value
            if isinstance(value, str):
                # work around Umlaut encodings like "M\\"unster"
                # and \S encoded as \\S
                found = False
                # see also https://github.com/WolfgangFahl/ProceedingsTitleParser/issues/38
                # remove encoded CR 
                for umlautTuple in [('\\"a', "ä"), ('\\"o', "ö"), ('\\"u', "ü"), ('\\', ' '), ('&#x0D;', '')]:
                    uc, u = umlautTuple
                    if uc in value:
                        value = value.replace(uc, u)
                        found = True  
                if found:
                    if debug:
                        print("Warning: fixing '%s' to '%s'" % (oldvalue, value))
                    eventInfo[key] = value   

    
class CrossrefEvent(Event):
    '''
    a scientific event derived from Crossref
    '''

    @classmethod
    def getSamples(cls):
        samples = [
            {
                "acronym": "SIGMIS CPR '06",
                "doi": "10.1145/1125170",
                "endDate": datetime.fromisoformat("2006-04-15"),
                "eventId": "10.1145/1125170",
                "location": "Claremont, California, USA",
                "month": 4,
                "name": "the 2006 ACM SIGMIS CPR conference",
                "number": "44",
                "source": "crossref",
                "sponsor": "SIGMIS, ACM Special Interest Group on Management Information Systems,ACM, Association for Computing Machinery",
                "startDate": datetime.fromisoformat("2006-04-13"),
                "theme": "Forty four years of computer personnel research: achievements, challenges & the future",
                "title": "Proceedings of the 2006 ACM SIGMIS CPR conference on computer personnel research Forty four years of computer personnel research: achievements, challenges & the future - SIGMIS CPR '06",
                "url": "https://api.crossref.org/v1/works/10.1145/1125170",
                "year": 2006
            }    
        ]
        return samples
    
    @classmethod
    def fromDOI(cls,doi:str):
        '''
        create a CrossRef Event from the given DOI
        
        Args:
           doi(str): the doi to get a CrossRef event for
        '''
        metadata=Crossref.doiMetaData(doi)
        event=CrossrefEvent()
        event.source='crossref'
        event.mapFromDict(metadata, [('doi','doi'),('ISBN','isbn'),('URL','url')])        
        if 'event' in metadata:
            eventdata=metadata['event']
            event.mapFromDict(eventdata,[('acronym','acronym'),('name','title')])          
        event.metadata=metadata
        return event
        

    
class CrossrefEventSeries(Event):
    '''
    a scientific event series derived from Crossref
    '''
      

class CrossrefEventManager(EventManager):
    '''
    Crossref event manager
    '''
        
    def __init__(self, config: StorageConfig=None):
        '''
        Constructor
        '''
        super().__init__(name="CrossrefEvents", sourceConfig=Crossref.sourceConfig, clazz=CrossrefEvent, config=config)
        
    def configure(self):
        '''
        configure me
        '''
        # nothing to do - there is a get ListOfDicts below    
        
    def getListOfDicts(self):
        '''
        cache my json files to my eventmanager
        '''
        startTime = time.time()
        jsonFiles = self.jsonFiles()
        lod = []
        if len(jsonFiles) < 50:
            print ("not enough crossref json files - did getsamples fail due to crossref API issues?")
            print ("need to download cached version of json files as a work-around instead")  #
            raise Exception(f"only {len(jsonFiles)} crossref json files available - there should be at least 47")
            # url="http://wiki.bitplan.com/images/confident/Event_crossref.db"
            # filename=self.em.getCacheFile(mode=StoreMode.SQL)
            # urllib.request.urlretrieve(url, filename)
        for jsonFilePath in jsonFiles:
            eventBatch = self.fromJsonFile(jsonFilePath)
            if self.debug:
                print("%4d: %s" % (len(eventBatch), jsonFilePath))
            for eventInfo in eventBatch:
                rawEvent = self.postProcess(eventInfo)
                lod.append(rawEvent)
                    
        if self.profile:
            elapsed = time.time() - startTime
            print (f"read {len(lod)} events in {elapsed:5.1f} s")
        return lod
        
    def postProcess(self, eventInfo:dict) -> dict:
        '''
        postProcess the given eventInfo e.g.
         {
        "event": {
          "name": "Adriatico Research Conference and Workshop",
          "location": "ICTP, Trieste, Italy"
        },
        "title": [
          "Towards the Theoretical Understanding of High Tc Superconductors"
        ],
        "DOI": "10.1142/0638"
      },

        '''
        rawEvent = eventInfo['event']
        rawEvent['source'] = "crossref"
        if 'title' in eventInfo:
            title = eventInfo["title"][0]
            rawEvent['title'] = title
            Ordinal.addParsedOrdinal(rawEvent)
        if 'sponsor' in eventInfo:
            sponsor = eventInfo['sponsor'][0]
            rawEvent['sponsor'] = sponsor    
        Crossref.fixEncodings(rawEvent, self.debug)
                            
        doi = eventInfo["DOI"]
        rawEvent['eventId'] = doi
        rawEvent['doi'] = doi
        if 'start' in rawEvent: 
            self.fixDateParts(rawEvent, 'start')
        if 'end'   in rawEvent: 
            self.fixDateParts(rawEvent, 'end')
        LOD.setNone(rawEvent, ['lookupAcronym', 'location', 'number', 'sponsor', 'month', 'year', 'startDate', 'endDate'])
        if 'year' in eventInfo:
            year = eventInfo["year"]
            if year is not None and type(year) is tuple:
                year = year[0]
                if not year in rawEvent:
                    rawEvent["year"] = int(year)
        rawEvent["url"] = f"https://api.crossref.org/v1/works/{doi}" 
        return rawEvent
        
    def fixDateParts(self, rawEvent:dict, key:str):
        '''
        fix date-parts from the json dict created by crossref e.g.
        "date-parts": [
              [
                1999,
                9,
                5
              ]
            ]
            
        Args:
            rawEvent(dict): the dictionary as parsed by json
            key(str): "start" or "end"

        '''
        if 'date-parts' in rawEvent[key]:
            # get the date-parts to be converted
            dateparts = rawEvent[key]['date-parts']
            # remove the original dict
            rawEvent.pop(key) 
            # get the tuple of values
            datetuple = tuple(dateparts[0])
            # set the year and month default
            year = None
            month = None
            # depending on the date tuples length we get more or less information
            if len(datetuple) == 3:
                # a fully specified date
                year, month, day = datetuple
                dt = datetime(year=year, month=month, day=day)
                date = dt.date()
                rawEvent[f"{key}Date"] = date
            elif len(datetuple) == 2:
                # year and month only
                year, month = datetuple
            elif len(datetuple) == 1:
                # year only
                year = datetuple[0] 
            else:
                if self.debug:
                    print(f"warning invalid date-tuple {str(datetuple)} found")
            if key == "start": 
                if year is not None: rawEvent["year"] = year
                if month is not None: rawEvent["month"] = month
    
    def jsonFiles(self) -> list: 
        '''
        get the list of the json files that have my data
        
        Return:
            list: a list of json file names
        
        '''
        cachePath = self.config.getCachePath()
        self.jsondir = f"{cachePath}/crossref"
        if not os.path.exists(self.jsondir):
                os.makedirs(self.jsondir)
        jsonFiles = sorted(glob.glob(f"{self.jsondir}/crossref-*.json"), key=lambda path:int(re.findall(r'\d+', path)[0]))
        return jsonFiles
    
    def fromJsonFile(self, jsonFilePath):
        '''
        get a single batch of events from the given jsonFilePath
        '''
        eventBatch = None
        with open(jsonFilePath) as jsonFile:
            response = json.load(jsonFile)  
            if 'status' in response:
                if response['status'] == 'ok':
                    eventBatch = response['message']['items']
        return eventBatch

    
class CrossrefEventSeriesManager(EventSeriesManager):
    '''
    CrossRef event series handling
    '''
    
    def __init__(self, config:StorageConfig=None):
        '''
        Constructor
        '''
        super().__init__(name="CrossrefEventSeries", sourceConfig=Crossref.sourceConfig, clazz=CrossrefEventSeries, config=config)

    def configure(self):
        '''
        configure me
        '''
        # nothing to do getListOfDicts is defined
        
    def getListOfDicts(self):
        '''
        get my data
        '''
        # TODO Replace this stub
        lod = [{'source':'crossref', 'eventSeriesId':'dummy'}]
        return lod



'''
Created on 2021-05-31

@author: wf
'''
from collections import Counter
from typing import Tuple, List

from lodstorage.tabulateCounter import TabulateCounter


class Tokenizer(object):
    '''
    categorize some text input
    '''

    def __init__(self, categories):
        self.categories = categories

    def tokenize(self, text, item):
        '''
        tokenize the given text for the given item
        '''
        tokenSequence = TokenSequence(text)
        tokenSequence.match(self.categories, item)
        return tokenSequence


class TokenSequence(object):
    '''
    a sequence of tokens
    '''

    def __init__(self, text: str, separator: str = ' '):
        '''
        constructor
        
        Args:
            separator(str): the separator for the tokens - default: blank
        '''
        self.text = text
        if text:
            self.words = text.split(separator)
        else:
            self.words = []
        self.matchResults: List['Token'] = []

    def next(self) -> Tuple[int, str]:
        """
        get the next token in this sequence

        Returns:
            (int, str): position of the token and the string representation of the token
        """
        for pos, word in enumerate(self.words):
            yield pos, self.words[pos]

    def match(self, categories: list, item: object) -> list:
        '''
        match me for the given categories
        
        Args:
            categories(list): the list of categories to match for
            item(object): the item this token sequence belongs to
        
        '''
        self.item = item
        for pos, tokenStr in self.next():
            for category in categories:
                if category.checkMatch(tokenStr) or (hasattr(category, "checkMatchWithContext")
                                                     and callable(getattr(category, "checkMatchWithContext"))
                                                     and category.checkMatchWithContext(tokenStr, pos, self)):
                    token = Token(category, self, pos, tokenStr, item)
                    self.matchResults.append(token)
        return self.matchResults

    def getTokenOfCategory(self, categoryName: str) -> List['Token']:
        """
        Returns list of matched tokens that are in the given category
        Args:
            categoryName: name of the category the tokens should be in

        Returns:
            list of tokens of the given category
        """
        tokens = [token for token in self.matchResults if token.name == categoryName]
        return tokens

    def __str__(self):
        text=f"{self.text}:{self.matchResults}"
        return text


class Token(object):
    '''
    a single categorized token
    '''

    def __init__(self, category: 'Category', tokenSequence: TokenSequence, pos: int, tokenStr: str, item):
        self.category = category
        self.name = category.name
        self.tokenSequence = tokenSequence
        self.pos = pos
        self.tokenStr = tokenStr
        self.value = category.add(item, tokenStr, pos, tokenSequence)

    def __str__(self):
        text = self.tokenStr
        return text

    def __repr__(self):
        return f"Token({self.__dict__})"


class Category(object):
    '''
    I am a category (a token type) representing an expected part of a signature
    e.g. a date an ordinal, a frequency
    '''

    def __init__(self, name, itemFunc):
        '''
        Constructor
        '''
        self.name = name
        self.items = {}
        self.itemFunc = itemFunc
        self.counter = Counter()
        self.subCategories = {}

    def checkMatch(self, tokenStr: str) -> bool:
        """
        Checks whether the given token lies within the category
        Args:
            tokenStr(str): value of the token

        Returns:
            True if the token is in the category otherwise False
        """
        return NotImplemented

    def checkMatchWithContext(self, tokenStr: str, pos: int, tokenSequence: TokenSequence) -> bool:
        """
        Checks whether the given token lies within the category
        Args:
            tokenStr(str): value of the token
            pos(int): position of the token
            tokenSequence: tokenSequence containing previously matched tokens

        Returns:
            True if the token is in the category otherwise False
        """
        return False

    def addCategory(self, category: 'Category'):
        self.subCategories[category.name] = category

    def add(self, item, propValue, pos:int, tokenSequence: TokenSequence):
        '''
        add the given item with the given value
        '''
        if self.itemFunc is not None:
            value = self.itemFunc(propValue)
        else:
            value = self.getValue(propValue, pos, tokenSequence)
        if value in self.items:
            self.items[value].append(item)
        else:
            self.items[value] = [item]
        self.counter[value] += 1
        return value

    def getValue(self, word: str, pos: int, tokenSequence: TokenSequence):
        """

        Args:
            word: raw token value
            pos: position of the raw token value
            tokenSequence: already matched tokens up to position pos-1

        Returns:
            category value of the token
        """
        return None

    def mostCommonTable(self, tablefmt='pretty', limit=50):
        '''
        get the most common Table
        '''
        tabulateCounter = TabulateCounter(self.counter)
        table = tabulateCounter.mostCommonTable(tablefmt=tablefmt, limit=limit)
        return table



'''
Created on 2022-02-16

@author: wf
'''
from corpus.eventcorpus import EventDataSource, EventDataSourceConfig
from corpus.event import EventSeriesManager,EventSeries, Event, EventManager
from lodstorage.storageconfig import StorageConfig
from corpus.datasources.tibkatftx import FTXParser
from corpus.utils.textparse import Textparse
import re
from corpus.utils.progress import Progress
from pathlib import Path
class Tibkat(EventDataSource):
    '''
    TIBKAT event meta data access
    
    https://www.tib.eu
    
    Technische Informationsbibliothek (TIB)
    
    Public datasets available via
    
    https://tib.eu/data/rdf
    
    '''
    sourceConfig = EventDataSourceConfig(lookupId="tibkat", name="tib.eu", url="https://www.tib.eu", title="TIBKAT", tableSuffix="tibkat",locationAttribute="location")
    home = str(Path.home())
    # use a symbolic link if you want a different location
    ftxroot=f"{home}/.conferencecorpus/tibkat/ftx"
    wantedbks=["54"] # Informatik
    limitFiles=10000
  
    def __init__(self):
        '''
        construct me
        '''
        super().__init__(TibkatEventManager(),TibkatEventSeriesManager(),Tibkat.sourceConfig)
  
class TibkatEventManager(EventManager):
    '''
    manage TIBKAT derived scientific events
    '''
    def __init__(self,config:StorageConfig=None):
        '''
        Constructor
        '''
        self.source="tibkat"
        super(TibkatEventManager,self).__init__(name="TIBKATEvents", sourceConfig=Tibkat.sourceConfig,clazz=TibkatEvent,config=config)
        
    def configure(self):
        '''
        configure me
        '''
    
    def isWantedBk(self,bk):
        '''
        check whether the given basisklassifikation bk is in the list of wanted ones
        
        Args:
            bk(str): the basisklassifkation to check
        '''
        for wantedbk in Tibkat.wantedbks:
            if bk.startswith(wantedbk):
                return True
        return False
        
    def isInWantedBkDocuments(self,document):
        '''
        filter for wanted Basisklassifikation
        
        Args:
            document(XMLEntity): the document to check
        '''
        if len(Tibkat.wantedbks)==0:
            return True
        wanted=False
        if hasattr(document, "bk"):
            bk=document.bk
            if isinstance(bk,list):
                for bkvalue in bk:
                    wanted=wanted or self.isWantedBk(bkvalue) 
            else:
                wanted=self.isWantedBk(bk)
        return wanted
         
    def getListOfDicts(self)->list:
        '''
        get my list of dicts
        '''
        lod=[]
        self.ftxParser=FTXParser(Tibkat.ftxroot)
        xmlFiles=self.ftxParser.ftxXmlFiles()
        xmlFiles=xmlFiles[:Tibkat.limitFiles]
        progress=Progress(progressSteps=1,expectedTotal=len(xmlFiles),msg=f"parsing {len(xmlFiles)} TIBKAT FTX files",showMemory=True)
        for xmlFile in xmlFiles:
            for document in self.ftxParser.parse(xmlFile,local=True):
                if self.isInWantedBkDocuments(document):
                    rawEvent=document.asDict()
                    TibkatEvent.postProcessLodRecord(rawEvent)
                    lod.append(rawEvent)
            progress.next()
        progress.done()
        return lod

class TibkatEvent(Event):
    '''
    event derived from TIBKAT
    '''
    
    def __init__(self):
        '''constructor '''
        super().__init__()
        pass
    
    @staticmethod
    def postProcessLodRecord(rawEvent:dict):
        '''
        fix the given raw Event
        
        Args:
            rawEvent(dict): the raw event record to fix
        '''
        rawEvent["source"]="tibkat"
        rawEvent["eventId"]=rawEvent["ppn"]
        if "description" in rawEvent:
            description=rawEvent["description"]
            if isinstance(description,list):
                parseResults=[]
                # find shortes Acronym
                for descEntry in description:
                    parseResult=TibkatEvent.parseDescription(descEntry)
                    if 'acronym' in parseResult and 'ordinal' in parseResult:
                        parseResults.append(parseResult)   
                parseResultsByAcronymLen = sorted(parseResults, key=lambda rawEvent: len(rawEvent['acronym']))             
                if len(parseResultsByAcronymLen)>0:
                    shortestAcronymDescriptionResult=parseResultsByAcronymLen[0]
                    TibkatEvent.mergeDict(rawEvent, shortestAcronymDescriptionResult)
            else:
                TibkatEvent.mergeDescription(description, rawEvent)
        pass
    
    @classmethod
    def mergeDict(cls,rawEvent:dict,more:dict):
        for key in more:
            rawEvent[key]=more[key]
    
    @classmethod
    def mergeDescription(cls,description:str,rawEvent:dict):
        parseResult=TibkatEvent.parseDescription(description)
        TibkatEvent.mergeDict(rawEvent, parseResult)
        
    @classmethod
    def parseDescription(cls,description:str)->dict:
        '''
        parse the given description
        
        Args:
            description(str): an event description
        '''
        result={}
        parts=description.split(";")
        if len(parts)>1:
            title=parts[0]
            titlepattern=r"\((?P<acronym>[^)]*)\)"
            titlematch=re.search(titlepattern,title)
            if titlematch:
                result["acronym"]=titlematch.group("acronym")
            else:
                result["acronym"]=title.strip()
            loctime=parts[1]
            #8 (Vietri) : 1996.05.23-25
            loctimepattern=r"\s?(?P<ordinal>[1-9][0-9]*)\s?\((?P<location>[^)]*)\)\s?:\s?(?P<daterange>[12][0-9][0-9][0-9]\.[0-9][0-9]\.[0-9][0-9]\-([0-9][0-9]\.[0-9][0-9]|[0-9][0-9]))"
            loctimematch=re.search(loctimepattern,loctime)
            if loctimematch:
                ordinalStr=loctimematch.group("ordinal")
                if ordinalStr is not None:
                    ordinal=int(ordinalStr)
                    # check for "year" ordinals
                    if ordinal>1000:
                        # completely ignore description 
                        return {}
                    result["ordinal"]=ordinal
                locationStr=loctimematch.group("location")
                if locationStr is not None:
                    result['location']=locationStr
                dateRangeStr=loctimematch.group("daterange")
                dateResult=Textparse.getDateRange(dateRangeStr)
                # merge with result
                result={**result, **dateResult}
            pass
            
        return result    
    
class TibkatEventSeriesManager(EventSeriesManager):
    '''
    TIBKAT event series access
     '''

    def __init__(self, config: StorageConfig=None):
        '''
        Constructor
        '''
        super().__init__(name="TibkatEventSeries", sourceConfig=Tibkat.sourceConfig, clazz=TibkatEventSeries, config=config)
        
    def configure(self):
        '''
        configure me
        '''
        
    def getListOfDicts(self)->list:
        '''
        get my list of dicts
        '''
        lod=[{"source":"tibkat"}]
        return lod
        
class TibkatEventSeries(EventSeries):
    '''
    a Tibkat Event Series
    
    '''

    def __init__(self):
        '''constructor '''
        super().__init__()
        pass



'''
Created on 2021-04-16

@author: wf
'''
from enum import Enum
from lodstorage.jsonable import JSONAble
from lodstorage.entity import EntityManager
from lodstorage.storageconfig import StorageConfig

class RatingType(str,Enum):
    '''
    the rating type
    
    see https://stackoverflow.com/a/51976841/1497139 why we use str as a type for better json Encoding
    '''
    missing='❌'
    invalid='👎'
    ok='👍'
    
class Rating(JSONAble):
    '''
    I am rating
    '''

    def __init__(self,pain:int=-1,reason:str=None,hint:str=None):
        '''
        Constructor
        
        Args:
            pain(int): the painLevel
            reason(str): the reason one of missing, invalid or ok
            hint(str): the description of the rating - hint on what is wrong
        '''
        self.set(pain,reason,hint)
        
    @classmethod
    def getSamples(cls):
        samplesLOD=[{
            "pain": 1,
            "reason": RatingType.ok,
            "hint": "Dates,  Jul 13, 2008 , Jul 14, 2008 valid",
        }
        ]
        return samplesLOD
        
    def set(self,pain:int,reason:str,hint:str):
        '''
        set my rating
        '''
        self.pain=pain
        self.reason=reason
        self.hint=hint
        
    def __str__(self):
        return f"{self.pain} - {self.reason}: {self.hint}"
    
class EntityRating(Rating):
    '''
    a rating for an entity
    '''
    
    def __init__(self,entity:object,entityType:str,entityId:str, source:str,pain:int=-1,reason:str=None,hint:str=None):
        '''
        construct me
        
        Args:
            entity(object): the entity to be rated
            entityId(str): the identifier for the entity
            source(str): the source of the entity
            entityType(str): the type of the entity
        '''
        super().__init__(pain,reason,hint)
        self.entity=entity
        self.entityType=entityType
        self.entityId=entityId
        self.source=source
        
    @classmethod
    def getSamples(cls):
        samplesLOD=[{
            "pain": 1,
            "reason": RatingType.invalid,
            "hint": "SPAM (locality=180)",
            "entityType": "Event",
            "source": "wikicfp",
            "entityId": "33127"
        }
        ]
        return samplesLOD
    
class RatingManager(EntityManager):
    '''
    a manager for Ratings
    '''
    
    def __init__(self,config:StorageConfig=None,debug=False):
        '''
        constructor
        '''
        name="Ratings"
        entityName="Rating"
        entityPluralName="Ratings"
        listName="ratings"
        clazz=EntityRating
        tableName="ratings"
        primaryKey=None
        super().__init__(name, entityName, entityPluralName, listName, clazz, tableName, primaryKey, config=config, debug=debug)
        


"""
Created on 2020-08-20

  script to read event metadata from http://wikicfp.com
  
  use: python3 wikicfp.py [startId] [stopId] [threads]
  @param startId
  @param stopId
  @param threads - number of threads the script should create to improve performance
  
  example: python3 wikicfp.py --startId 2000 --stopId 2999 10

  @author:     svantje, wf
  @copyright:  2020-2021 TIB Hannover, Wolfgang Fahl. All rights reserved.

"""
from corpus.datasources.webscrape import WebScrape
from corpus.event import EventStorage,EventManager, EventSeriesManager
import datetime
from enum import Enum
import glob
import re
import os
import sys
import threading
import time
from argparse import ArgumentParser
from argparse import RawDescriptionHelpFormatter
#from lodstorage.jsonpicklemixin import JsonPickleMixin
from lodstorage.storageconfig import StorageConfig
import corpus.datasources.wikicfp as wcfp
#import jsonpickle

class CrawlType(Enum):
    '''
    possible supported storage modes
    '''
    EVENT = "Event"      
    SERIES = "Series"
    
    @property
    def urlPrefix(self):
        baseUrl="http://www.wikicfp.com/cfp"
        if self.value is CrawlType.EVENT.value:
            url= f"{baseUrl}/servlet/event.showcfp?eventid="
        elif self.value is CrawlType.SERIES.value:
            url= f"{baseUrl}/program?id="
        return url
    
 
    @classmethod
    def isValid(cls,value:str)->bool:
        '''
        check whether the given value is valid
        
        Args:
            value(str): the value to check
        
        Return:
            bool: True if the value is a valid value of this enum
        '''
        return value in ["Event","Series"]

    @classmethod
    def ofValue(cls,value:str):
        if value=="Event":
            return CrawlType.EVENT
        elif value=="Series":
            return CrawlType.SERIES
        else:
            return None

class CrawlBatch(object):
    '''
    describe a batch of pages to fetch metadata from
    '''
    
    def __init__(self,threads:int,startId:int,stopId:int,crawlTypeValue:str,threadIndex=None,):
        '''
        construct me with the given number of threads, startId and stopId
        
        Args:
           threads(int): number of threads to use
           startId(int): id of the event to start crawling with
           stopId(int): id of the event to stop crawling
           crawlTypeValue(str): the type of crawling (Event or Series)
           threadIndex(int): the index of this batch
        '''
        self.threads=threads
        self.threadIndex=threadIndex
        self.startId=startId
        self.stopId=stopId     
        if startId <= stopId: 
            self.step = +1
            self.total = stopId-startId+1
        else: 
            self.step = -1
            self.total = startId-stopId+1
        self.batchSize = self.total // threads
        if not CrawlType.isValid(crawlTypeValue):
            raise Exception(f"Invalid crawlType {crawlTypeValue}")
        self.crawlType=CrawlType.ofValue(crawlTypeValue)
        
    def split(self)->list:
        '''
        split me for my threads
        '''
        crawlBatches=[]
        for threadIndex in range(self.threads):
            s = self.startId + threadIndex * self.batchSize
            e = s + self.batchSize-1
            splitBatch=CrawlBatch(1,s, e,self.crawlType.value,threadIndex)
            crawlBatches.append(splitBatch)
        return crawlBatches
      
    
    def __str__(self):
        '''
        get my string representation
        Return
            str: my string representation
        '''
        text=f"WikiCFP {self.crawlType.value} IDs {self.startId} - {self.stopId} ({self.threads} threads of {self.batchSize} IDs each)"
        return text
 
class WikiCfpScrape(object):
    '''
    support events from http://www.wikicfp.com/cfp/
    '''

    def __init__(self,jsonEventManager,jsonEventSeriesManager,profile:bool=True,debug:bool=False,jsondir:str=None,limit=200000,batchSize=1000,showProgress=True):
        '''
        Constructor
        
        Args:
            config(StorageConfig): the storage configuration to use
            debug(bool): if True debug for crawling is on
            limit(int): maximum number of entries to be crawled
            batchSize(int): default size of batches
            showProgress(bool): if True show Progress
        '''
        self.debug=debug
        self.limit=limit
        self.batchSize=batchSize
        self.showProgress=showProgress
        self.jsonEventManager=jsonEventManager
        self.jsonEventSeriesManager=jsonEventSeriesManager
        self.jsonManagers={
            "Event": jsonEventManager,
            "Series": jsonEventSeriesManager
        }
        self.profile=profile
        if jsondir is not None:
            self.jsondir=jsondir
        else:
            cachePath=self.jsonEventManager.config.getCachePath()
            self.jsondir=f"{cachePath}/wikicfp"
            if not os.path.exists(self.jsondir):
                    os.makedirs(self.jsondir)
        
    def getManager(self,crawlType:CrawlType):
        '''
        get the manager for the given crawlType
        
        Args:
           crawlType(CrawlType) the manager for the given CrawlType
        '''
        manager=self.jsonManagers[crawlType.value]
        return manager
    
    def cacheToJsonManager(self,crawlType:CrawlType):
        '''
        cache the crawlType entities to my json manager for that crawlType
        
        Args:
            crawlType(CrawlType): the CrawlType to work with
            
        Return:
            EntityBasemanager: the entityManager
            
        '''
        jsonEm=self.getManager(crawlType)
        if jsonEm.isCached():
            jsonEm.fromStore()
        else:    
            self.crawlFilesToJson(crawlType=crawlType,withStore=True)
        return jsonEm    
        
    def crawlFilesToJson(self,crawlType:CrawlType,withStore=True):    
        '''
        convert the crawlFiles to Json
        
        Args:
            crawlType(CrawlType): the CrawlType to work with
            withStore(bool): True if the data should be stored after collecting
        
        Return:
            jsonEm(EventBaseManager): the JSON - storage based EventBaseManager to use to collect the results
        '''
        # crawling is not done on startup but need to be done
        # in command line mode ... we just collect the json crawl result files here
        #self.crawl(startId=startId,stopId=stopId)
        jsonEm=self.getManager(crawlType)
        entityList=jsonEm.getList()
        startTime=time.time()
        jsonFiles=self.jsonFiles(crawlType)
        if len(jsonFiles)==0:
            if self.profile or self.debug:
                print(f"No wikiCFP crawl json backups for {crawlType.value} available")
        else:
            # loop over all crawled files
            for jsonFilePath in jsonFiles:
                config=StorageConfig.getJSON(debug=self.debug)
                config.cacheFile=jsonFilePath
                if crawlType.value is CrawlType.EVENT.value:
                    batchEm=EventManager(name=jsonFilePath,clazz=jsonEm.clazz,config=config)
                elif crawlType.value is CrawlType.SERIES.value:
                    batchEm=EventSeriesManager(name=jsonFilePath,clazz=jsonEm.clazz,config=config)
                # legacy mode -file were created before ORM Mode
                # was available - TODO: make new files available in ORM mode with jsonable
                #jsonPickleEm=JsonPickleMixin.readJsonPickle(jsonFileName=jsonFilePath,extension='.json')
                #jsonPickleEvents=jsonPickleEm['events']
                #if self.showProgress:
                #    print("%4d: %s" % (len(jsonPickleEvents),jsonFilePath))
                batchEm.fromStore(cacheFile=jsonFilePath)
                for entity in batchEm.getList():
                    if hasattr(entity, "startDate"):
                        if entity.startDate is not None:
                            entity.year=entity.startDate.year  
                    doAdd=not(entity.deleted)
                    # SPAM Filter
                    if hasattr(entity, "locality"):
                        if entity.locality.startswith("1"):
                            doAdd=False
                    # Series Filter
                    if hasattr(entity, "title"):
                        if entity.title.startswith("WikiCFP : Call For Papers of Conferences, Workshops and Journals"):
                            doAdd=False
                    
                    if doAdd:
                        entityList.append(entity)
            if self.profile:
                elapsed=time.time()-startTime
                print (f"read {len(entityList)} {crawlType.value} records in {elapsed:5.1f} s")
            if withStore:
                jsonEm.store(limit=self.limit,batchSize=self.batchSize)
        return jsonEm
        
    def jsonFiles(self,crawlType:CrawlType)->list:  
        '''
        get the list of the json files that have my data
        
        Args:
            crawlType(CrawlType): the tpe of the files
            
        Return:
            list: a list of json file names
        '''
        jsonFiles=sorted(glob.glob(f"{self.jsondir}/wikicfp_{crawlType.value}*.json"),key=lambda path:int(re.findall(r'\d+',path)[0]))
        return jsonFiles    
        
    def getJsonFileName(self,crawlBatch):
        '''
        get my the JsonFileName 
        
        Args:
            crawlBatch(CrawlBatch): the batch to crawl):
            
        Return:
            str: the json file name for this batch
        '''
        jsonFilePath=f"{self.jsondir}/wikicfp_{crawlBatch.crawlType.value}{crawlBatch.startId:06d}-{crawlBatch.stopId:06d}.json"
        return jsonFilePath
    
    def getBatchEntityManager(self,crawlBatch:CrawlBatch):
        '''
        get the batch Entity Manager for the given crawlBatch
        
        Args:
            crawlBatch(CrawlBatch): the batch to crawl):
            
        Return:
            EntityManager: either a Event or a Series Manager
        '''
        jsonFilepath=self.getJsonFileName(crawlBatch)
        config=EventStorage.getStorageConfig(debug=self.debug, mode="json")
        config.cacheFile=jsonFilepath
        crawlType=crawlBatch.crawlType
        print(f"CrawlBatch has crawlType {type(crawlType)}{crawlType}/{crawlType.value}")
        if crawlType.value is CrawlType.EVENT.value:
            batchEm=wcfp.WikiCfpEventManager(config=config)
        elif crawlType.value is CrawlType.SERIES.value:
            batchEm=wcfp.WikiCfpEventSeriesManager(config=config)
        else:
            raise Exception(f"Invalid crawlType {crawlType}")
        return batchEm
        
    def crawl(self,crawlBatch:CrawlBatch):
        '''
        see https://github.com/TIBHannover/confIDent-dataScraping/blob/master/wikicfp.py
        
        Args:
            crawlBatch(CrawlBatch): the batch to crawl
        '''
       
        print(f'crawling {crawlBatch}')
        batchEm=self.getBatchEntityManager(crawlBatch)
 
        # get all ids
        crawlType=crawlBatch.crawlType
        for eventId in range(int(crawlBatch.startId), int(crawlBatch.stopId+1), crawlBatch.step):
            wEvent=WikiCfpEventFetcher(crawlType=crawlType)
            retry=1
            maxRetries=3
            retrievedResult=False
            while not retrievedResult:
                try:
                    rawEvent=wEvent.fromEventId(eventId)
                    if crawlType.value is CrawlType.EVENT.value:
                        event=wcfp.WikiCfpEvent()
                        event.fromDict(rawEvent)
                        title="? deleted: %r" %event.deleted if not 'title' in rawEvent else event.title
                        batchEm.getList().append(event)
                    elif crawlType.value is CrawlType.SERIES.value:
                        eventSeries=wcfp.WikiCfpEventSeries()
                        eventSeries.fromDict(rawEvent)
                        title="?" if not 'title' in rawEvent else eventSeries.title
                        batchEm.getList().append(eventSeries)
                    retrievedResult=True
                except Exception as ex:
                    if "HTTP Error 500" in str(ex):
                        print(f"{eventId} inaccessible due to HTTP Error 500")
                        retrievedResult=True
                    elif "timed out" in str(ex):
                        print(f"{eventId} access timed Out on retry attempt {retry}")
                        retry+=1
                        if retry>maxRetries:
                            raise ex
                    else:
                        raise ex
                    pass
                
            print(f"{eventId:06d}: {title}")
           
           
        batchEm.store()
        return batchEm
            
    def threadedCrawl(self,crawlBatch:CrawlBatch):
        '''
        crawl with the given number of threads, startId and stopId
        
        Args:
            crawlBatch(CrawlBatch): the batch to crawl
        '''
        # determine the eventId range for each threaded job
        startTime=time.time()
        
        msg=f'Crawling {crawlBatch}'
        print(msg)

        # this list will contain all threads -> we can wait for all to finish at the end
        jobs = []

        # now start each thread with its id range and own filename
        for crawlBatch in crawlBatch.split(): 
        
            thread = threading.Thread(target = self.crawl, args=(crawlBatch,))
            jobs.append(thread)
            
        for job in jobs:
            job.start()   

        # wait till all threads have finished before print the last output
        for job in jobs:
            job.join()

        if self.debug:
            elapsed=time.time()-startTime
            print(f'crawling done after {elapsed:5.1f} s')
               
      
class WikiCfpEventFetcher(object):
    '''
    a single WikiCfpEentFetcher to fetch and event or series
    '''
    def __init__(self,crawlType=CrawlType.EVENT,debug=False,showProgress:bool=True,timeout=20):
        '''
        construct me
        
        Args:
            showProgress(bool): if True show progress
            timeout(float): the default timeout
        
        '''
        self.debug=debug
        self.crawlType=crawlType
        self.showProgress=showProgress
        self.progressCount=0
        self.timeout=timeout
            
    def fromTriples(self,rawEvent,triples): 
        '''
        get the rawEvent dict from the given triple e.g.:
        
        v:Event(v:summary)=IDC 2009
        v:Event(v:eventType)=Conference
        v:Event(v:startDate)=2009-06-03T00:00:00
        v:Event(v:endDate)=2009-06-05T23:59:59
        v:Event(v:locality)=Milano, Como, Italy
        v:Event(v:description)= IDC  2009 : The 8th International Conference on Interaction Design and Children
        v:Address(v:locality)=Milano, Como, Italy
        v:Event(v:summary)=Submission Deadline
        v:Event(v:startDate)=2009-01-19T00:00:00
        v:Event(v:summary)=Notification Due
        v:Event(v:startDate)=2009-02-20T00:00:00
        v:Event(v:summary)=Final Version Due
        v:Event(v:startDate)=2009-03-16T00:00:00
        '''
        recentSummary=None
        for s,p,o in triples:
            s=s.replace("v:","")
            p=p.replace("v:","")
            if self.debug:
                print ("%s(%s)=%s" % (s,p,o)) 
            if recentSummary in ['Submission Deadline','Notification Due','Final Version Due']:             
                key=recentSummary.replace(" ","_")
            else:
                key=p 
            if p.endswith('Date'):
                dateStr=o
                if dateStr=="TBD":
                    o=None
                else:    
                    o=datetime.datetime.strptime(
                        dateStr, "%Y-%m-%dT%H:%M:%S").date()
            if not key in rawEvent: 
                rawEvent[key]=o    
            if p=="summary": 
                recentSummary=o 
            else: 
                recentSummary=None
                
    @staticmethod       
    def getUrl(cfpid,crawlType:CrawlType=CrawlType.EVENT)->str:
        '''
        Args:
            cfpid(int): the WikiCFP id of the event or series
            crawlType(CrawlType): Event or Series
            
        Returns:
            the WikiCfP url
        '''
        url=f"{crawlType.urlPrefix}{cfpid}"
        return url   
    
    @classmethod
    def getLatestEvent(cls,debug=False,showProgress=True):
        '''
        get the latest Event doing a binary search
        '''
        wikicfp=WikiCfpEventFetcher(debug,showProgress=showProgress)
        wikicfp.progressCount=0
        wikicfp.getLatesEvetFromPair()
        
    def getHighestNonDeletedIdInRange(self,fromId:int,toId:int)->int:
        '''
        get the event with the highest id int the range fromId to toId that is not deleted
        
        Args:
            fromId(int): minimum id to search from
            toId(int): maximum id to search to
            
        Returns:
            int: maxium id of an event that is not deleted or None if there is none in this range
        '''
        maxId=None
        for eventId in range(fromId,toId+1):
            if self.showProgress:
                print(".",end='',flush=True)
                self.progressCount+=1
                if self.progressCount % 50 == 0:
                    print(flush=True)
            rawEvent=self.fromEventId(eventId)
            if not rawEvent['deleted']:
                maxId=eventId
        return maxId
    
    def getLatesEvetFromPair(self,low=5000,high=300000,margin=40):
        '''
        get the latest Event doing a binary search
        
        Args:
            low(int): lower index to search from
            hight(int): upper index boundary
        '''
        if high-low>margin+1:
            mid=(high+low)//2
            midId=(self.getHighestNonDeletedIdInRange(mid-margin, mid))
            if midId:
                return self.getLatesEvetFromPair(mid+1,high)
            else:
                return self.getLatesEvetFromPair(low, mid-1)
        else:
            return mid
        pass
                           
    def fromEventId(self,cfpid:int):
        '''
        see e.g. https://github.com/andreeaiana/graph_confrec/blob/master/src/data/WikiCFPCrawler.py
        
        Args:
            cfpid(int): the wikicfp id to use
        '''
        url=WikiCfpEventFetcher.getUrl(cfpid,self.crawlType)
        return self.fromUrl(url)
    
    def rawEventFromWebScrape(self,rawEvent:dict,triples:list,scrape:WebScrape):
        '''
        fill the given rawEvent with the data derived from the scrape 
        
        Args:
            rawEvent(dict): the event dictionary
            triples(list): the triples found
            scrape(WebScrape): the webscrape object to be used for parsing
        '''
        if len(triples)==0:
            #scrape.printPrettyHtml(scrape.soup)
            firstH3=scrape.fromTag(scrape.soup, 'h3')
            if "This item has been deleted" in firstH3:
                rawEvent['deleted']=True
        else:        
            self.fromTriples(rawEvent,triples)
            # add series information
            # Tag: <a href="/cfp/program?id=1769&amp;s=ISWC&amp;f=International Semantic Web Conference">International Semantic Web Conference</a>
            m,seriesText=scrape.findLinkForRegexp(r'/cfp/program\?id=([0-9]+).*')
            if m:
                seriesId=m.group(1)
                rawEvent['seriesId']=seriesId
                rawEvent['series']=seriesText
                pass
                
            if 'summary' in rawEvent:
                rawEvent['acronym']=rawEvent.pop('summary').strip()
            if 'description' in rawEvent:
                rawEvent['title']=rawEvent.pop('description').strip()
                
    def rawEventSeriesFromWebScrape(self,rawEvent:dict,scrape:WebScrape):
        '''
        fill the given rawEventSeries with the data derived from the scrape 
        
        Args:
            rawEvent(dict): the event dictionary
            scrape(WebScrape): the webscrape object to be used for parsing
        '''
        title=scrape.soup.find("title")
        rawEvent["title"]=title.text.strip()
        dblpM,_text=scrape.findLinkForRegexp(r'http://dblp.uni-trier.de/db/([a-zA-Z0-9/-]+)/index.html')
        if dblpM:
            dblpSeriesId=dblpM.group(1)
            rawEvent['dblpSeriesId']=dblpSeriesId
        pass
 
    
    def fromUrl(self,url:str)->dict:
        '''
        get the event form the given url
        
        Args:
            url(str): the url to get the event from
        
        Returns:
            dict: a raw event dict or None if an error occured
        
        '''
        regexp=r"^"+self.crawlType.urlPrefix.replace("?","\?")+"(\d+)$"
        m=re.match(regexp,url)
        if not m:
            raise Exception("Invalid URL %s" % (url))
        else:
            cfpId=int(m.group(1))
        rawEvent={}
        if self.crawlType.value is CrawlType.EVENT.value:
            rawEvent['eventId']=f"{cfpId}" 
        elif self.crawlType.value is CrawlType.SERIES.value:
            rawEvent['seriesId']=f"{cfpId}" 
        rawEvent['url']=url
        rawEvent['wikiCfpId']=cfpId
        rawEvent['deleted']=False
        scrape=WebScrape(debug=self.debug,timeout=self.timeout)
        triples=scrape.parseRDFa(url)
        if scrape.err:
            raise Exception(f"fromUrl {url} failed {scrape.err}")
        if self.crawlType.value is CrawlType.EVENT.value:
            self.rawEventFromWebScrape(rawEvent, triples, scrape)
        else:
            self.rawEventSeriesFromWebScrape(rawEvent,scrape)
        return rawEvent
    
__version__ = 0.4
__date__ = '2020-06-22'
__updated__ = '2021-08-09'    

DEBUG = 1

    
def main(argv=None): # IGNORE:C0111
    '''main program.'''

    if argv is None:
        argv=sys.argv[1:]
        
    program_name = os.path.basename(__file__)
    program_version = "v%s" % __version__
    program_build_date = str(__updated__)
    program_version_message = '%%(prog)s %s (%s)' % (program_version, program_build_date)
    program_shortdesc = "script to read event metadata from http://wikicfp.com"
    user_name="Wolfgang Fahl"
    program_license = '''%s

  Created by %s on %s.
  Copyright 2020-2021 TIB Hannover, Wolfgang Fahl. All rights reserved.

  Licensed under the Apache License 2.0
  http://www.apache.org/licenses/LICENSE-2.0

  Distributed on an "AS IS" basis without warranties
  or conditions of any kind, either express or implied.

USAGE
''' % (program_shortdesc,user_name, str(__date__))

    try:
        # Setup argument parser
        parser = ArgumentParser(description=program_license, formatter_class=RawDescriptionHelpFormatter)
        parser.add_argument("-d", "--debug", dest="debug", action="count", help="set debug level [default: %(default)s]")
        parser.add_argument('-V', '--version', action='version', version=program_version_message)
        parser.add_argument('--startId', type=int, help='eventId to start crawling from', required=True)
        parser.add_argument('--stopId', type=int, help='eventId to stop crawling at', required=True)
        parser.add_argument('--crawlType',type=str,default="Event",help="The crawlType - Event or Series")
        parser.add_argument('-p','--targetPath',type=str,help="targetPath (JSON directory) for crawl results")
        parser.add_argument('-t','--threads', type=int, help='number of threads to start', default=10)

        # Process arguments
        args = parser.parse_args(argv)
        wikiCfp=wcfp.WikiCfp()
        wikiCfpScrape=wikiCfp.wikiCfpScrape
        wikiCfpScrape.jsondir=args.targetPath
        wikiCfpScrape.debug=args.debug
        crawlBatch=CrawlBatch(args.threads, args.startId, args.stopId,args.crawlType,None)
        wikiCfpScrape.threadedCrawl(crawlBatch)
        
    except KeyboardInterrupt:
        ### handle keyboard interrupt ###
        return 1
    except Exception as e:
        if DEBUG:
            raise(e)
        indent = len(program_name) * " "
        sys.stderr.write(program_name + ": " + repr(e) + "\n")
        sys.stderr.write(indent + "  for help use --help")
        return 2     
    
if __name__ == "__main__":
    if DEBUG:
        sys.argv.append("-d")
    sys.exit(main())



'''
Created on 2021-08-05

@author: wf
'''

class EventDataSourceConfig(object):
    '''
    holds configuration parameters for an EventDataSource
    '''
    def __init__(self,lookupId:str,name:str,title:str,url:str,tableSuffix:str,locationAttribute:str=None):
        '''
        constructor 
        
        Args:
          lookupId(str): the id of the data source
          name(str): the name of the data source
          title(str): the title of the data source
          url(str): the link to the data source homepage
          tableSuffix(str): the tableSuffix to use
          locationAttribute(str): the location Attribute to use
        '''  
        self.lookupId=lookupId
        self.name=name
        self.title=title
        self.url=url
        self.tableSuffix=tableSuffix
        self.locationAttribute=locationAttribute
        
    def getTableName(self,entityName:str):
        '''
        Args:
            entityName(str): the name of the entity
        Return:
            str: the tablename
        '''
        tableName=f"{entityName.lower()}_{self.tableSuffix}"
        return tableName
   
        


'''
Created on 2022-04-11

@author: wf
'''
from pyparsing import oneOf
from ptp.parsing import Tokenizer
from ptp.signature import ParsingCategory, RegexpCategory, OrdinalCategory, EnumCategory, CountryCategory, YearCategory, \
    VolumeCategory, CityPrefixCategory, CityCategory, AcronymCategory


class EventReferenceParser(object):
    '''
    given an Event reference returns the signature with probabilities
    '''


    def __init__(self):
        '''
        Constructor
        '''
        self.categories=[
            RegexpCategory("first Letter",lambda word:word[0] if word else '',r".*"),
            RegexpCategory("word",lambda word:word,r".*"),
            # TODO: region, city
            OrdinalCategory(),
            YearCategory(),
            CountryCategory(),
            EnumCategory('month'),
            EnumCategory('delimiter'),
            EnumCategory('eventType'),
            EnumCategory('extract'),
            EnumCategory('field'),
            EnumCategory('frequency'),
            EnumCategory('organization'),
            EnumCategory('publish'),
            EnumCategory('scope'),
            EnumCategory('syntax'),
            ParsingCategory('part',"Part"+oneOf("A B C 1 2 3 4 I II III IV")+"."),
            VolumeCategory(),
            CityPrefixCategory(),
            CityCategory(),
            AcronymCategory()
        ]
        self.tokenizer=Tokenizer(self.categories)
        
    def parse(self,eventReference:str,eventContext:object,show:bool=True):
        '''
        parse the given eventReference for the given eventContext
        
        Args:
            eventReference(str): a reference for an event
            eventContext(object): the object this reference is referencing
            show(bool): if True show the parse result
        '''
        tokenSequence=self.tokenizer.tokenize(eventReference,eventContext)
        if show:
            for token in tokenSequence.matchResults:
                print(token)
        return tokenSequence
        
    def showStatistics(self,tablefmt="mediawiki"):
        '''
        show the statistics in the given table format
        '''
        for category in self.categories:
            print(f"=== {category.name} ===")
            print(category.mostCommonTable(tablefmt=tablefmt))
            
    def lookupToYaml(self,lookup,name:str,tables:list,yamlPath:str,show:bool=True):
        '''
        save the given lookup dictionary as a yamlfile
        
        Args:
            lookup(dict): the lookup dictionary
            name(str): the name of the dictionary
            tables(list): the source tables this lookup is created from
            yamlPath(str): the path where to store the dictionary
            show(bool): if True show details before saving
        '''
        if show:
            print(f"found {len(lookup)} {name}")
        yamlstr=f"""# lookup for {name}
# created from {len(lookup)} {name} 
# analyzed from the sources {tables}
"""
        for entry in lookup:
            d=lookup[entry]
            yamlstr+=f"""{d["name"]}:
    qid: {d["qid"]}
    frequency: {d["frequency"]}\n"""
            for table in tables:
                if table in d:
                    yamlstr+=f"    {table}: {d[table]}\n"
        yamlFile=f"{yamlPath}/{name}.yaml"
        print(yamlstr,  file=open(yamlFile, 'w'))
        


'''
Created on 2022-03-03

@author: wf
'''
from corpus.utils.download import Download
from os import path
import os
import urllib
from corpus.xmlhandler.xmlparser import XMLEntityParser
from corpus.utils.progress import Progress

class DROPS(object):
    '''
    access to Dagstuhl research online publication server
    
    '''

    def __init__(self,maxCollectionId:int):
        '''
        Constructor
        
        Args: 
          maxCollectionId(int): the maximum collectionId currently published in DROPS
        '''
        self.maxCollectionId=maxCollectionId
        home = path.expanduser("~")
        self.cachedir= f"{home}/.conferencecorpus/drops"
        if not os.path.exists(self.cachedir):
            os.makedirs(self.cachedir)
        
    def xmlFilepath(self,collectionId):
        '''
        get my xmlFilepath
        
        Returns:
            str: the path to my xml file in the cache directory
        '''
        xmlp=f"{self.cachedir}/{collectionId}.xml"
        return xmlp
        
    def cache(self,collectionId,baseurl="https://submission.dagstuhl.de/services/metadata/xml/collections",force:bool=False,progress:Progress=None):
        '''
        cache the XML file for the given collectionId
        
        Args:
            collectionId(int): the id of the volume
            baseurl(str): the base url
            force(bool): if true reload even if already cached
            progressStep(int): if > 0 show the progress with numeric display every progressStep items
        '''
      
        cfilepath=self.xmlFilepath(collectionId)
        if Download.needsDownload(cfilepath,force):
            url= f"{baseurl}/{collectionId}"
            try:
                xml=Download.getURLContent(url)
                with open(cfilepath, "w") as xmlfile:
                    xmlfile.write(xml)
                    if progress is not None:
                        progress.next()
            except urllib.error.HTTPError as err:
                if not "HTTP Error 404: Not Found" in str(err):
                    raise err
                pass
                
    def parse(self,collectionId:int,progress:Progress=None):
        '''
          parse the xml data of  volume with the given collectionId
          
          Args:
            collectionId(int): the id of the volume
            baseurl(str): the base url
            force(bool): if true reload even if already cached
            progressStep(int): if > 0 show the progress with numeric display every progressStep items
        '''    
        recordTag="{https://submission.dagstuhl.de/services/metadata/xml/dagpub.xsd}volume"
        xmlPath=self.xmlFilepath(collectionId)
        namespaces={'ns0':'https://submission.dagstuhl.de/services/metadata/xml/dagpub.xsd'}
        xmlPropertyMap= {
            "title": './ns0:title',
            "shortTitle": './ns0:shortTitle',
            "date": './ns0:date',
            "location": './ns0:location',
            'dblp': './ns0:conference/ns0:dblp',
            'website': '.ns0:conference/ns0:website'
        }        
        if os.path.exists(xmlPath):
            xmlParser=XMLEntityParser(xmlPath,recordTag)
            for xmlEntity in xmlParser.parse(xmlPropertyMap,namespaces):
                yield(xmlEntity)
                if progress is not None:
                    progress.next()


import numpy as np
import calendar
from datetime import datetime, timedelta, date
from typing import List, Union

from corpus.utils.plots import Plot


class DatePolarHistogram(Plot):
    """
    polar histogram for dates
    only month and date are plotted the year is ignored

    see https://matplotlib.org/2.0.2/examples/pie_and_polar_charts/polar_bar_demo.html
    """

    bottom = 0

    def __init__(self, dates: List[Union[datetime, date]], title:str=None):
        """
        constructor

        Args:
            dates(list): list of dates the dates need to be a pared date object
            title(str): title of the plot
        """
        self.setup(title)
        hist={monthDay:0 for monthDay in self.__getBaseNumbers()}
        for date in dates:
            monthDay = self.getMonthDateNumber(date)
            if monthDay in hist:
                hist[monthDay] += 1
        data = np.array(list(hist.values()))
        N = len(hist)
        theta = np.linspace(0.0, 2 * np.pi, N, endpoint=False)
        radii = data
        width = (2*np.pi) / N
        self.ax = self.plt.subplot(polar=True)
        self.bars = self.ax.bar(theta, radii, width=width, bottom=self.bottom)
        positions, labels = self.__get_x_ticks()
        positions = [theta[i] for i in positions]
        self.plt.xticks(positions, labels)

    def __get_x_ticks(self) -> (List[int], List[str]):
        """
        Generates the ticks and labels for the x-axis

        Returns:
            (positions, labels) position and label of the ticks
        """
        position = []
        labels = []
        day = 0
        for i in range(1,13):
            position.append(day)
            labels.append(calendar.month_name[i])
            days_in_month = calendar.monthrange(2020, i)[1]
            day += days_in_month
        return position, labels

    @staticmethod
    def getMonthDateNumber(date: Union[datetime, date]) -> int:
        """
        Converts the given date into a number based on month and date
        Format: monthDate
        Example:
            2022-03-05 → 305
            2022-10-10 → 1010

        Args:
            date: date to convert

        Returns:
            int monthDate
        """
        day = date.day
        if day <10:
            day = f'0{day}'
        else:
            day = f"{day}"
        month = str(date.month)
        return int(f"{month}{day}")

    @staticmethod
    def gen_days_of_year(year: int) -> List[datetime]:
        """
        Generates a list of all dates within the given year
        see https://stackoverflow.com/questions/32507287/how-to-generate-range-of-dates-every-day-of-a-given-year
        Args:
            year: year for which all dates should be generated

        Returns:
            list of all dates in the given year
        """
        start_date=datetime( year, 1, 1 )
        end_date=datetime( year, 12, 31 )
        d=start_date
        dates=[ start_date ]
        while d < end_date:
            d += timedelta(days=1)
            dates.append( d )
        return dates

    def __getBaseNumbers(self) -> List[int]:
        """
        Returns:
             list of all possible numbers on the DatePolarHistogram spectrum
        """
        leapyear = 2020  # does not matter which
        return [DatePolarHistogram.getMonthDateNumber(date) for date in DatePolarHistogram.gen_days_of_year(leapyear)]


'''
Created on 2022-03-05

@author: wf
'''
import os
import sys
import traceback
from lodstorage.lod import LOD
from corpus.version import Version
from corpus.lookup import CorpusLookup, CorpusLookupConfigure
from corpus.locationfixer import LocationFixer
from corpus.datasources.dblpxml import DblpXml
from corpus.datasources.tibkat import Tibkat
from corpus.utils.download import Profiler
from argparse import ArgumentParser
from argparse import RawDescriptionHelpFormatter
from corpus.event import EventStorage
from ptp.eventrefparser import EventReferenceParser

class ConferenceCorpusUpdate():
    '''
    updater for the Conference Corpus database
    
    allows updating single or multiple datasources of the conferencecorpus
    e.g. via command line
    '''
    def __init__(self,lookupId:str, debug:bool=False):
        '''
        constructor
        
        Args:
            lookupId(str): the lookupId of the dataSource to be used
            debug(bool): If True show debug messages
        '''
        self.debug=debug
        self.lookupId=lookupId
        
    def getEventDataSource(self,forceUpdate:bool=False):
        '''
        get my event data source
        
        Args:    
            forceUpdate(bool): if True force updating the event data source
        
        Returns:
            EventDataSource: the event data source for my lookup id
        '''
        lookup=CorpusLookup(lookupIds=[self.lookupId], debug=self.debug)
        lookup.load(forceUpdate=forceUpdate,showProgress=True)
        eventDataSource=lookup.getDataSource(self.lookupId)
        return eventDataSource
        
    def updateDataSource(self,source:str,sampleId:str=None):
        '''
        update the given DataSource
        
        Args:
            source(str): the name of the datasource to update
            sampleId(str): the id of the sample record to display when done
        '''
        msg=f"update of conference corpus database from {source}"
        profiler=Profiler(msg)
        eventDataSource=self.getEventDataSource(forceUpdate=True)
        el=eventDataSource.eventManager.getList()
        esl=eventDataSource.eventSeriesManager.getList()
        msg=f"{eventDataSource.name}: {len(el)} events {len(esl)} eventseries"
        profiler.time(msg)
        eventsByAcronym,_dup=LOD.getLookup(el, "acronym")
        if sampleId is not None:
            if sampleId in eventsByAcronym:
                event=eventsByAcronym[sampleId]
                print (event.toJSON())
            else:
                print(f"sample event id {sampleId} not found")
                
    def addLookupAcronyms(self):
        '''
        add lookup acronyms
        '''
        msg=f"adding lookup acronyms for {self.lookupId}"
        profiler=Profiler(msg)
        eventDataSource=self.getEventDataSource(forceUpdate=False)
        el=eventDataSource.eventManager.getList()
        for event in el:
            event.getLookupAcronym()
        eventDataSource.eventManager.store()
        msg=f"{eventDataSource.name}: added lookupAcronym for {len(el)} events"
        profiler.time(msg)
        
    
class TibkatUpdater(ConferenceCorpusUpdate):
    '''
    update for Tibkat data from FTX XML dump
    '''
    
    def __init__(self):
        '''
        constructor
        '''
        super().__init__("tibkat")
        
    def update(self,args):
        '''
        update
        
        Args:
            args: command line arguments
        '''
        if args.updateConferenceCorpus or args.updateAll:
            Tibkat.limitFiles=args.limitFiles
            Tibkat.ftxroot=args.ftxroot
            Tibkat.wantedbks=args.wantedBks
            if Tibkat.wantedbks==["all"]:
                Tibkat.wantedbks=[]
            super().updateDataSource(source=f"TIBKAT FTX dump \nftxroot:{args.ftxroot}\nlimitFiles:{args.limitFiles}\nBasisklassifikationen: {args.wantedBks}",sampleId=args.sample)
        

class DblpUpdater(ConferenceCorpusUpdate):
    '''
    updater for Dblp data from XML dump
    '''
    
    def __init__(self):
        '''
        constructor
        '''
        super().__init__("dblp")
            
    def update(self,args):
        '''
        update
        
        Args:
            args: command line arguments
        '''
        debug=args.debug
        dblpXml=DblpXml(debug=debug)
        reload=args.updateXml or args.updateAll
        xmlfile=dblpXml.getXmlFile(reload=reload)
        sizeMB=dblpXml.getSize()/1024/1024
        print(f"dblp xml dump file is  {xmlfile} with size {sizeMB:5.1f} MB" )
        if args.updateXml or args.updateAll:
            showProgress=True
            recreate=True
            limit=10000000
            sample=5
            _sqlDB=dblpXml.getSqlDB(limit, sample=sample, debug=debug,recreate=recreate,postProcess=dblpXml.postProcess,showProgress=showProgress)
        if args.updateConferenceCorpus or args.updateAll:
            super().updateDataSource("dblp xml dump","ISWC 2008") 

def main(argv=None): # IGNORE:C0111
    '''main program.'''

    if argv is None:
        argv=sys.argv[1:]
        
    program_name = os.path.basename(__file__)
    program_version = "v%s" % Version.version
    program_build_date = str(Version.updated)
    program_version_message = '%%(prog)s %s (%s)' % (program_version, program_build_date)
    program_shortdesc = "Conference Corpus update from command line"
    user_name="Wolfgang Fahl/Tim Holzheim"
    program_license = '''%s

  Created by %s on %s.
  Copyright 2022 contributors. All rights reserved.

  Licensed under the Apache License 2.0
  http://www.apache.org/licenses/LICENSE-2.0

  Distributed on an "AS IS" basis without warranties
  or conditions of any kind, either express or implied.

USAGE
''' % (program_shortdesc,user_name, str(Version.date))

    try:
        # Setup argument parser
        parser = ArgumentParser(description=program_license, formatter_class=RawDescriptionHelpFormatter)
        parser.add_argument("-d",   "--debug", dest="debug", action="store_true", help="set debug [default: %(default)s]")
        parser.add_argument("--createViews",action="store_true",help="create the common view for all datasources")
        parser.add_argument("--createLookup",action="store_true",help="create lookup yaml files for city,country and region for the given table prefixes")
        parser.add_argument("--lookupTables",nargs="+",default=["dblp","wikidata","crossref","confref"],help="tables to use for lookup Creation\n[default: %(default)s]")
        parser.add_argument("-dblp","--dblp", dest="dblp",   action="store_true", help="update dblp")
        parser.add_argument("--tibkat", action="store_true",help="update tibkat from ftx")
        parser.add_argument("--fixlocations",nargs="+",help="fix the locations for the given lookup Ids")
        parser.add_argument("--perCentLimit",type=float,default=60.0,help="limit the percentage of Locations to be covered [default: %(default)s]")
        parser.add_argument("--addLookupAcronym",nargs="+",help="add lookupAcronyms for the given lookup Ids")
        parser.add_argument("--updateSource",nargs="+",help="update the sources for the given lookup Ids")
        parser.add_argument("--limitFiles",type=int,default=10000,help="limit the number of file to be parsed [default: %(default)s]")
        parser.add_argument("--ftxroot",default="/Volumes/seel/tibkat-ftx/tib-intern-ftx_0/tib-2021-12-20",help="path to root directory of ftx xml files [default: %(default)s]")
        parser.add_argument("--sample",default="ISWC 2008",help="sample event ID [default: %(default)s]")
        parser.add_argument("--wantedBks",nargs="+",default=["54"],help="wanted Basisklassifikationen [default: %(default)s] use 'all' for no filter")
        parser.add_argument("-uxml","--updateXml", dest="updateXml",   action="store_true", help="update the dblp xml file and eventcorpus database")
        parser.add_argument("-ucc","--updateConferenceCorpus", dest="updateConferenceCorpus",   action="store_true", help="update the dblp xml file and eventcorpus database")
        parser.add_argument("-uall","--updateAll", dest="updateAll",   action="store_true", help="update the dblp xml file and eventcorpus database")
        parser.add_argument('-V', '--version', action='version', version=program_version_message)
        
        args = parser.parse_args(argv)
        if args.debug:
            DEBUG = True
            print("Starting in debug mode")
        if args.dblp:
            dblpUpdater=DblpUpdater()
            dblpUpdater.update(args)
        if args.tibkat:
            tibkatUpdater=TibkatUpdater()
            tibkatUpdater.update(args)
        if args.fixlocations:
            locationFixer=LocationFixer()
            locationFixer.fixLocations4LookupIds(args.fixlocations,args.perCentLimit)
        if args.updateSource:
            for lookupId in args.updateSource:
                updater=ConferenceCorpusUpdate(lookupId, debug=args.debug)
                updater.updateDataSource(f"{lookupId} cache", args.sample)
        if args.addLookupAcronym:
            for lookupId in args.addLookupAcronym:
                updater=ConferenceCorpusUpdate(lookupId)
                updater.addLookupAcronyms()
        if args.createLookup:
            tables=args.lookupTables
            eParser=EventReferenceParser()
            yamlPath="/tmp"
            for column,columnPlural in [("country","countries"),("city","cities"),("region","regions")]:
                lookup=EventStorage.createLookup(column,tables)
                eParser.lookupToYaml(lookup, columnPlural, tables, yamlPath,show=True)
            
        if args.createViews:
            profiler=Profiler("Creating common views")
            EventStorage.createViews(exclude=EventStorage.viewTableExcludes,show=True)
            storageTableList=EventStorage.getTableList()
            print(f"found {len(storageTableList)} storage Tables for the ConferenceCorpus")
            for baseEntity in ["Event","EventSeries"]:
                plantUml=EventStorage.asPlantUml(baseEntity,exclude=EventStorage.viewTableExcludes)
                print(plantUml)
            profiler.time()
            
            
    except KeyboardInterrupt:
        ### handle keyboard interrupt ###
        return 1
    except Exception as e:
        if DEBUG:
            raise(e)
        indent = len(program_name) * " "
        sys.stderr.write(program_name + ": " + repr(e) + "\n")
        sys.stderr.write(indent + "  for help use --help")
        if args.debug:
            print(traceback.format_exc())
        return 2     
        
DEBUG = 0
    
if __name__ == "__main__":
    if DEBUG:
        sys.argv.append("-d")
    sys.exit(main())
